@@ -1,4 +1,5 @@
 /*
+ *  Copyright (c) 2014, Peter Haag
  *  Copyright (c) 2009, Peter Haag
  *  Copyright (c) 2004-2008, SWITCH - Teleinformatikdienste fuer Lehre und Forschung
  *  All rights reserved.
@@ -11,7 +12,7 @@
  *   * Redistributions in binary form must reproduce the above copyright notice, 
  *     this list of conditions and the following disclaimer in the documentation 
  *     and/or other materials provided with the distribution.
- *   * Neither the name of SWITCH nor the names of its contributors may be 
+ *   * Neither the name of the author nor the names of its contributors may be 
  *     used to endorse or promote products derived from this software without 
  *     specific prior written permission.
  *  
@@ -71,10 +72,6 @@
 #include "util.h"
 #include "flist.h"
 
-/* Global vars */
-extern char *CurrentIdent;
-
-
 /*
  * Select a single file
  * --------------------
@@ -223,9 +220,11 @@ static struct entry_filter_s {
 
 #define NUM_PTR 16
 
+// globals
+extern uint32_t	twin_first, twin_last;
+
 static char		*first_file, *last_file;
 static char		*current_file = NULL;
-static uint32_t	twin_first, twin_last;
 static stringlist_t source_dirs, file_list;
 
 /* Function prototypes */
@@ -799,6 +798,9 @@ printf("DGB: short fts: '%s', filer_first: '%s', filter_last: '%s'\n",
 					continue;
 				if ( strstr(ftsent->fts_name, ".stat") != NULL )
 					continue;
+				// skip pcap file
+				if ( strstr(ftsent->fts_name, "pcap") != NULL )
+					continue;
 
 				if ( file_list_level && (
 					( fts_level != file_list_level ) ||
@@ -888,8 +890,6 @@ char	path[MAXPATHLEN];
 } // End of Getsource_dirs
 
 void SetupInputFileSequence(char *multiple_dirs, char *single_file, char *multiple_files) {
-char *error;
-int	fd;
 
 	twin_first  = 0;
 	twin_last   = 0xffffffff;
@@ -909,29 +909,33 @@ int	fd;
 
 		// get time window spanning all the files 
 		if ( file_list.num_strings ) {
-			stat_record_t *stat_ptr;
+			stat_record_t stat_ptr;
 
-			fd = OpenFile(file_list.list[0], &stat_ptr, &error);	// read the stat record
-			if ( error != NULL ) {
-				fprintf(stderr, "%s\n", error);
+			// read the stat record
+			if ( !GetStatRecord(file_list.list[0], &stat_ptr) ) {
 				exit(250);
 			}
-			close(fd);
-			twin_first = stat_ptr->first_seen;
-			fd = OpenFile(file_list.list[file_list.num_strings-1], &stat_ptr, &error);	// read the stat record of last file
-			if ( error != NULL ) {
-				fprintf(stderr, "%s\n", error);
+			twin_first = stat_ptr.first_seen;
+
+			// read the stat record of last file
+			if ( !GetStatRecord(file_list.list[file_list.num_strings-1], &stat_ptr) ) {
 				exit(250);
 			}
-			close(fd);
-			twin_last  = stat_ptr->last_seen;
+			twin_last  = stat_ptr.last_seen;
 		}
 
 	} else if ( single_file ) {
 		CleanPath(single_file);
 
 		if ( source_dirs.num_strings == 0 ) {
-				InsertString(&file_list, single_file);
+			stat_record_t stat_ptr;
+			InsertString(&file_list, single_file);
+			if ( !GetStatRecord(single_file, &stat_ptr) ) {
+				exit(250);
+			}
+			twin_first = stat_ptr.first_seen;
+			twin_last  = stat_ptr.last_seen;
+
 		} else {
 			int i;
 
@@ -951,9 +955,15 @@ int	fd;
 						// file not found - try to guess subdir
 						char *sub_dir = GuessSubDir(source_dirs.list[i], single_file);
 						if ( sub_dir ) {	// subdir found
+							stat_record_t stat_ptr;
 							snprintf(s, MAXPATHLEN-1, "%s/%s/%s", source_dirs.list[i], sub_dir, single_file);
 							s[MAXPATHLEN-1] = '\0';
 							InsertString(&file_list, s);
+							if ( !GetStatRecord(s, &stat_ptr) ) {
+								exit(250);
+							}
+							twin_first = stat_ptr.first_seen;
+							twin_last  = stat_ptr.last_seen;
 						} else {	// no subdir found
 							fprintf(stderr, "stat() error '%s': %s\n", s, "File not found!");
 						}
@@ -965,7 +975,14 @@ int	fd;
 					if ( !S_ISREG(stat_buf.st_mode) ) {
 						fprintf(stderr, "Skip non file entry: '%s'\n", s);
 					} else {
+						stat_record_t stat_ptr;
 						InsertString(&file_list, s);
+						if ( !GetStatRecord(s, &stat_ptr) ) {
+							exit(250);
+						}
+						twin_first = stat_ptr.first_seen;
+						twin_last  = stat_ptr.last_seen;
+				
 					}
 				}
 			}
@@ -980,66 +997,50 @@ char *GetCurrentFilename(void) {
 	return current_file;
 } // End of GetCurrentFilename
 
-int GetNextFile(int current, time_t twin_start, time_t twin_end, stat_record_t **stat_record) {
-stat_record_t *stat_ptr;
-char *error;
-int fd;
+nffile_t *GetNextFile(nffile_t *nffile, time_t twin_start, time_t twin_end) {
 static int cnt;
 
-	// is it first time init ?
-	if ( current < 0 ) {
-		cnt  = 0;
-	}
-
 	// close current file before open the next one
 	// stdin ( current = 0 ) is not closed
-	if ( current > 0 ) {
-		close(current);
+	if ( nffile ) {
+		CloseFile(nffile);
 		current_file = NULL;
+	} else {
+		// is it first time init ?
+		cnt  = 0;
 	}
 
 	// no or no more files available
 	if ( file_list.num_strings == cnt ) {
-		if ( stat_record )
-			*stat_record = NULL;
 		current_file = NULL;
 		return EMPTY_LIST;
 	}
 	
-/*
-	while ( cnt < file_list.num_strings ) {
-		printf("Process: '%s'\n", file_list.list[cnt++]);
-	}
-*/
 
 	while ( cnt < file_list.num_strings ) {
-		fd = OpenFile(file_list.list[cnt], &stat_ptr, &error);	// Open the file
+#ifdef DEVEL
+		printf("Process: '%s'\n", file_list.list[cnt] ? file_list.list[cnt] : "<stdin>");
+#endif
+		nffile = OpenFile(file_list.list[cnt], nffile);	// Open the file
+		if ( !nffile ) {
+			return NULL;
+		}
 		current_file = file_list.list[cnt];
 		cnt++;
 
 		// stdin
-		if ( fd == STDIN_FILENO ) {
-			if ( stat_record )
-				*stat_record = NULL;
-			CurrentIdent = "none";
+		if ( nffile->fd == STDIN_FILENO ) {
 			current_file = NULL;
-			return fd;
+			return nffile;
 		}
 
-		if ( fd > 0 && CheckTimeWindow(twin_start, twin_end, stat_ptr) ) {
+		if ( CheckTimeWindow(twin_start, twin_end, nffile->stat_record) ) {
 			// printf("Return file: %s\n", string);
-			if ( stat_record ) 
-				*stat_record = stat_ptr;
-			return fd;
+			return nffile;
 		} 
-		close(fd);
-		if ( error != NULL ) 
-			fprintf(stderr, "%s\n", error);
+		CloseFile(nffile);
 	}
 
-	if ( stat_record )
-		*stat_record = NULL;
-
 	current_file = NULL;
 	return EMPTY_LIST;
 
