@@ -1,5 +1,6 @@
 /*
  *  All rights reserved.
+ *  Copyright (c) 2016, Peter Haag
  *  Copyright (c) 2014, Peter Haag
  *  Copyright (c) 2009, Peter Haag
  *  Copyright (c) 2004-2008, SWITCH - Teleinformatikdienste fuer Lehre und Forschung
@@ -32,16 +33,11 @@
  *  
  *  Flow-Tools related code taken from flow-tools-0.67 cretated by Mark Fullmer
  *
- *  $Author: haag $
- *
- *  $Id: ft2nfdump.c 69 2010-09-09 07:17:43Z haag $
- *
- *  $LastChangedRevision: 69 $
- *	
- *
  */
 
+#ifdef HAVE_CONFIG_H 
 #include "config.h"
+#endif
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -66,6 +62,7 @@
 #include "nffile.h"
 #include "nfx.h"
 #include "launch.h"
+#include "util.h"
 
 /* Global defines */
 #define MAXRECORDS 30
@@ -91,7 +88,7 @@ void usage(char *name);
 
 extension_info_t *GenExtensionMap(struct ftio *ftio);
 
-int flows2nfdump(struct ftio *ftio, extension_info_t *extension_info, int extended, uint32_t limitflows);
+int flows2nfdump(struct ftio *ftio, char *wfile, int compress, extension_info_t *extension_info, int extended, uint32_t limitflows);
 
 #define NEED_PACKRECORD
 #include "nffile_inline.c"
@@ -104,9 +101,12 @@ void usage(char *name) {
 					"-c\t\tLimit number of records to convert.\n"
 					"-m\t\tPrint the extension map and exit.\n"
 					"-V\t\tPrint version and exit.\n"
-					"-r\t\tread input from file\n"
+					"-r <file>\tread flow-tools records from file\n"
+					"-w <file>\twrite nfdump records to file\n"
+					"-j\t\tBZ2 compress flows in output file.\n"
+					"-z\t\tLZO compress flows in output file.\n"
 					"Convert flow-tools format to nfdump format:\n"
-					"ft2nfdump -r <flow-tools-data-file> | nfdump -z -w <nfdump-file>\n"
+					"ft2nfdump -r <flow-tools-data-file> -w <nfdump-file> [-z]\n"
 				, name);
 
 } // End of usage
@@ -187,7 +187,7 @@ int	i;
 
 } // End of GenExtensionMap
 
-int flows2nfdump(struct ftio *ftio, extension_info_t *extension_info, int extended, uint32_t limitflows) {
+int flows2nfdump(struct ftio *ftio, char *wfile, int compress, extension_info_t *extension_info, int extended, uint32_t limitflows) {
 // required flow tools variables
 struct fttime 		ftt;
 struct fts3rec_offsets fo;
@@ -200,7 +200,7 @@ char				*s;
 uint32_t			cnt;
 
 	s = "flow-tools";
-	nffile = OpenNewFile( "-", NULL, 0, 0, s);
+	nffile = OpenNewFile( wfile , NULL, compress, 0, s);
 	if ( !nffile ) {
 		fprintf(stderr, "%s\n", s);
 		return 1;
@@ -316,8 +316,8 @@ struct ftio ftio;
 extension_info_t *extension_info;
 struct stat statbuf;
 uint32_t	limitflows;
-int i, extended, printmap, ret, fd;
-char   *ftfile;
+int i, extended, printmap, ret, fd, compress;;
+char   *ftfile, *wfile;
 
 	/* init fterr */
 	fterr_setid(argv[0]);
@@ -326,8 +326,10 @@ char   *ftfile;
 	printmap 	= 0;
 	limitflows 	= 0;
 	ftfile   	= NULL;
+	wfile		= "-";
+	compress 	= NOT_COMPRESSED;
 
-	while ((i = getopt(argc, argv, "EVc:hmr:?")) != -1)
+	while ((i = getopt(argc, argv, "jzEVc:hmr:w:?")) != -1)
 		switch (i) {
 			case 'h': /* help */
 				case '?':
@@ -356,6 +358,14 @@ char   *ftfile;
 				printmap = 1;
 				break;
 
+			case 'j':
+				compress = LZO_COMPRESSED;
+				break;
+
+			case 'z':
+				compress = BZ2_COMPRESSED;
+				break;
+
 			case 'r':
 				ftfile = optarg;
 				if ( (stat(ftfile, &statbuf) < 0 ) || !(statbuf.st_mode & S_IFREG) ) {
@@ -363,6 +373,10 @@ char   *ftfile;
 					exit(255);
 				}
 				break;
+
+			case 'w':
+				wfile = optarg;
+				break;
 		
 			default:
 				usage(argv[0]);
@@ -398,7 +412,7 @@ char   *ftfile;
 		exit(255);
 	} 
 
-	ret = flows2nfdump(&ftio, extension_info, extended, limitflows);
+	ret = flows2nfdump(&ftio, wfile, compress, extension_info, extended, limitflows);
 
 	return ret;
 
