@@ -1,4 +1,6 @@
 /*
+ *  Copyright (c) 2016, Peter Haag
+ *  Copyright (c) 2014, Peter Haag
  *  Copyright (c) 2009, Peter Haag
  *  Copyright (c) 2004-2008, SWITCH - Teleinformatikdienste fuer Lehre und Forschung
  *  All rights reserved.
@@ -11,7 +13,7 @@
  *   * Redistributions in binary form must reproduce the above copyright notice, 
  *     this list of conditions and the following disclaimer in the documentation 
  *     and/or other materials provided with the distribution.
- *   * Neither the name of SWITCH nor the names of its contributors may be 
+ *   * Neither the name of the author nor the names of its contributors may be 
  *     used to endorse or promote products derived from this software without 
  *     specific prior written permission.
  *  
@@ -27,26 +29,21 @@
  *  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
  *  POSSIBILITY OF SUCH DAMAGE.
  *  
- *  $Author: haag $
- *
- *  $Id: launch.c 69 2010-09-09 07:17:43Z haag $
- *
- *  $LastChangedRevision: 69 $
- *	
- *
  */
 
+#ifdef HAVE_CONFIG_H 
 #include "config.h"
+#endif
 
 #include <signal.h>
 #include <sys/types.h>
 #include <sys/time.h>
 #include <sys/resource.h>
 #include <sys/wait.h>
+#include <sys/socket.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#include <syslog.h>
 #include <stdarg.h>
 #include <errno.h>
 #include <unistd.h>
@@ -73,6 +70,7 @@
 #include "nffile.h"
 #include "nfxstat.h"
 #include "collector.h"
+#include "util.h"
 
 static int done, launch, child_exit;
 
@@ -151,7 +149,7 @@ int  i;
 					s = ident;
 					break;
 				default:
-					syslog(LOG_ERR, "Unknown format token '%%%c'\n", q[i+1]);
+					LogError("Unknown format token '%%%c'", q[i+1]);
 					s = NULL;
 			}
 			if ( s ) {
@@ -163,7 +161,7 @@ int  i;
 				// be a bit paranoid and prevent endless expansion
 				if ( strlen(q) > MAXCMDLEN ) {
 					// this is fishy
-					syslog(LOG_ERR, "Error: cmdline too long!\n");
+					LogError("Error: cmdline too long!");
 					return NULL;
 				}
 				memmove(&q[i] + strlen(s), &q[i+2], strlen(&q[i+2]) + 1);   // include trailing '0' in memmove
@@ -213,7 +211,7 @@ int i, argnum;
 	if ( (i >= MAXCMDLEN) || (argnum >= MAXARGS) ) {
 		// for safety reason, disable the command
     	args[0] = NULL;	
-		syslog(LOG_ERR, "Launcher: Unable to parse command: '%s'", buf);
+		LogError("Launcher: Unable to parse command: '%s'", buf);
 	}
 
 } // End of cmd_parse
@@ -226,20 +224,20 @@ static void cmd_execute(char **args) {
 int pid;
 
     // Get a child process.
-	syslog(LOG_DEBUG, "Launcher: fork child.");
+	LogInfo("Launcher: fork child.");
 	if ((pid = fork()) < 0) {
-		syslog(LOG_ERR, "Can't fork: %s", strerror(errno));
+		LogError("Can't fork: %s", strerror(errno));
         return;
 	}
 
     if (pid == 0) {	// we are the child
         execvp(*args, args);
-		syslog(LOG_ERR, "Can't execvp: %s: %s", args[0], strerror(errno));
+		LogError("Can't execvp: %s: %s", args[0], strerror(errno));
         _exit(1);
     }
 
 	// we are the parent
-	syslog(LOG_DEBUG, "Launcher: child exec done.");
+	LogInfo("Launcher: child exec done.");
 	/* empty */
 
 } // End of cmd_execute
@@ -249,7 +247,7 @@ bookkeeper_t 	*books;
 dirstat_t 		*dirstat, oldstat;
 int				ret, bookkeeper_stat, do_rescan;
 
-	syslog(LOG_INFO, "Run expire on '%s'", datadir);
+	LogInfo("Run expire on '%s'", datadir);
 
 	do_rescan = 0;
 	ret = ReadStatInfo(datadir, &dirstat, CREATE_AND_LOCK);
@@ -259,16 +257,16 @@ int				ret, bookkeeper_stat, do_rescan;
 		case ERR_NOSTATFILE:
 			dirstat->low_water = 95;
 		case FORCE_REBUILD:
-			syslog(LOG_INFO, "Force rebuild stat record");
+			LogInfo("Force rebuild stat record");
 			do_rescan = 1;
 			break;
 		case ERR_FAIL:
-			syslog(LOG_ERR, "expire failed: can't read stat record");
+			LogError("expire failed: can't read stat record");
 			return;
 			/* not reached */
 			break;
 		default:
-			syslog(LOG_ERR, "expire failed: unexpected return code %i reading stat record", ret);
+			LogError("expire failed: unexpected return code %i reading stat record", ret);
 			return;
 			/* not reached */
 	}
@@ -288,15 +286,15 @@ int				ret, bookkeeper_stat, do_rescan;
 		UpdateBookStat(dirstat, &tmp_books);
 		ReleaseBookkeeper(books, DETACH_ONLY);
 	} else {
-		syslog(LOG_ERR, "Error %i: can't access book keeping records", ret);
+		LogError("Error %i: can't access book keeping records", ret);
 	}
 
-	syslog(LOG_INFO, "Limits: Filesize %s, Lifetime %s, Watermark: %llu%%\n", 
+	LogInfo("Limits: Filesize %s, Lifetime %s, Watermark: %llu%%\n", 
 		dirstat->max_size     ? ScaleValue(dirstat->max_size)    : "<none>", 
 		dirstat->max_lifetime ? ScaleTime(dirstat->max_lifetime) : "<none>",
 		(unsigned long long)dirstat->low_water);
 		
-	syslog(LOG_INFO, "Current size: %s, Current lifetime: %s, Number of files: %llu",
+	LogInfo("Current size: %s, Current lifetime: %s, Number of files: %llu",
 		ScaleValue(dirstat->filesize),
 		ScaleTime(dirstat->last - dirstat->first),
 		(unsigned long long)dirstat->numfiles);
@@ -307,16 +305,16 @@ int				ret, bookkeeper_stat, do_rescan;
 	WriteStatInfo(dirstat);
 
 	if ( (oldstat.numfiles - dirstat->numfiles) > 0 ) {
-		syslog(LOG_INFO, "expire completed");
-		syslog(LOG_INFO, "   expired files: %llu", (unsigned long long)(oldstat.numfiles - dirstat->numfiles));
-		syslog(LOG_INFO, "   expired time slot: %s", ScaleTime(dirstat->first - oldstat.first));
-		syslog(LOG_INFO, "   expired file size: %s", ScaleValue(oldstat.filesize - dirstat->filesize));
-		syslog(LOG_INFO, "New size: %s, New lifetime: %s, Number of files: %llu",
+		LogInfo("expire completed");
+		LogInfo("   expired files: %llu", (unsigned long long)(oldstat.numfiles - dirstat->numfiles));
+		LogInfo("   expired time slot: %s", ScaleTime(dirstat->first - oldstat.first));
+		LogInfo("   expired file size: %s", ScaleValue(oldstat.filesize - dirstat->filesize));
+		LogInfo("New size: %s, New lifetime: %s, Number of files: %llu",
 			ScaleValue(dirstat->filesize),
 		ScaleTime(dirstat->last - dirstat->first),
 			(unsigned long long)dirstat->numfiles);
 	} else {
-		syslog(LOG_INFO, "expire completed - nothing to expire.");
+		LogInfo("expire completed - nothing to expire.");
 	}
 	ReleaseStatInfo(dirstat);
 
@@ -331,7 +329,7 @@ srecord_t	*InfoRecord;
 
 	InfoRecord = (srecord_t *)commbuff;
 
-	syslog(LOG_INFO, "Launcher: Startup. auto-expire %s", expire ? "enabled" : "off" );
+	LogInfo("Launcher: Startup. auto-expire %s", expire ? "enabled" : "off" );
 	done = launch = child_exit = 0;
 
 	// process may be NULL, if we only expire data files
@@ -349,7 +347,7 @@ srecord_t	*InfoRecord;
 		while ( fs ) {
 			cmd = cmd_expand(&TestRecord, fs->Ident, fs->datadir, process);
 			if ( cmd == NULL ) {
-				syslog(LOG_ERR, "Launcher: ident: %s, Unable to expand command: '%s'", fs->Ident, process);
+				LogError("Launcher: ident: %s, Unable to expand command: '%s'", fs->Ident, process);
 				exit(255);
 			}
 
@@ -369,9 +367,9 @@ srecord_t	*InfoRecord;
 
 	while ( !done ) {
 		// sleep until we get signaled
-		syslog(LOG_DEBUG, "Launcher: Sleeping");
+		dbg_printf("Launcher: Sleeping");
 		select(0, NULL, NULL, NULL, NULL);
-		syslog(LOG_DEBUG, "Launcher: Wakeup");
+		dbg_printf("Launcher: Wakeup");
 		if ( launch ) {	// SIGHUP
 			launch = 0;
 
@@ -383,11 +381,10 @@ srecord_t	*InfoRecord;
 					// Expand % placeholders
 					cmd = cmd_expand(InfoRecord, fs->Ident, fs->datadir, process);
 					if ( cmd == NULL ) {
-						syslog(LOG_ERR, "Launcher: ident: %s, Unable to expand command: '%s'", fs->Ident, process);
+						LogError("Launcher: ident: %s, Unable to expand command: '%s'", fs->Ident, process);
 						continue;
 					}
-					// printf("Launcher: run command: '%s'\n", cmd);
-					syslog(LOG_DEBUG, "Launcher: ident: %s run command: '%s'", fs->Ident, cmd);
+					dbg_printf("Launcher: ident: %s run command: '%s'", fs->Ident, cmd);
 
 					// prepare args array
 					cmd_parse(cmd, args);
@@ -410,29 +407,29 @@ srecord_t	*InfoRecord;
 			}
 		}
 		if ( child_exit ) {
-			syslog(LOG_INFO, "laucher child exit %d childs.", child_exit);
+			LogInfo("launcher child exit %d children.", child_exit);
 			while ( (pid = waitpid (-1, &stat, WNOHANG)) > 0  ) {
 				if ( WIFEXITED(stat) ) {
-					syslog(LOG_DEBUG, "launcher child %i exit status: %i", pid, WEXITSTATUS(stat));
+					LogInfo("launcher child %i exit status: %i", pid, WEXITSTATUS(stat));
 				}
 				if (  WIFSIGNALED(stat) ) {
-					syslog(LOG_WARNING, "laucher child %i died due to signal %i", pid, WTERMSIG(stat));
+					LogError("launcher child %i died due to signal %i", pid, WTERMSIG(stat));
 				}
 
 				child_exit--;
 			}
-			syslog(LOG_INFO, "laucher waiting childs done. %d childs", child_exit);
+			LogInfo("launcher waiting children done. %d children", child_exit);
 			child_exit = 0;
 		}
 		if ( done ) {
-			syslog(LOG_INFO, "Launcher: Terminating.");
+			LogInfo("Launcher: Terminating.");
 		}
 	}
 
 	waitpid (-1, &stat, 0);
 
 	// we are done
-	syslog(LOG_INFO, "Launcher: exit.");
+	LogInfo("Launcher: exit.");
 
 } // End of launcher
 
