@@ -1,4 +1,5 @@
 /*
+ *  Copyright (c) 2016, Peter Haag
  *  Copyright (c) 2014, Peter Haag
  *  Copyright (c) 2009, Peter Haag
  *  Copyright (c) 2004-2008, SWITCH - Teleinformatikdienste fuer Lehre und Forschung
@@ -28,22 +29,17 @@
  *  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
  *  POSSIBILITY OF SUCH DAMAGE.
  *  
- *  $Author: peter $
- *
- *  $Id: netflow_v1.c 30 2011-07-18 11:19:46Z peter $
- *
- *  $LastChangedRevision: 30 $
- *	
  */
 
+#ifdef HAVE_CONFIG_H 
 #include "config.h"
+#endif
 
 #include <stdio.h>
 #include <sys/types.h>
 #include <sys/socket.h>
 #include <unistd.h>
 #include <stdlib.h>
-#include <syslog.h>
 #include <netinet/in.h>
 #include <arpa/inet.h>
 #include <errno.h>
@@ -64,12 +60,6 @@
 #include "exporter.h"
 #include "netflow_v1.h"
 
-#ifndef DEVEL
-#   define dbg_printf(...) /* printf(__VA_ARGS__) */
-#else
-#   define dbg_printf(...) printf(__VA_ARGS__)
-#endif
-
 extern int verbose;
 extern extension_descriptor_t extension_descriptor[];
 
@@ -129,20 +119,21 @@ uint16_t	map_size;
 	// default map - 0 extensions
 	map_size 		 = sizeof(extension_map_t);
 	i=0;
+	dbg_printf("v1 map: map size start: %u\n", map_size);
 	while ( (id = v1_full_map[i]) != 0  ) {
 		if ( extension_descriptor[id].enabled ) {
 			extension_size += extension_descriptor[id].size;
 			map_size += sizeof(uint16_t);
+			dbg_printf("v1 map: enabled extension %u\n", id);
 		}
 		i++;
 	}
+	dbg_printf("v1 map: map size so far: %u\n", map_size);
+
 	// extension_size contains the sum of all optional extensions
 	// caculate the record size 
 	v1_output_record_size = COMMON_RECORD_DATA_SIZE + V1_BLOCK_DATA_SIZE + extension_size;  
  
-	// now the full extension map size
-	map_size	+= sizeof(extension_map_t);
- 
 	// align 32 bits
 	if ( ( map_size & 0x3 ) != 0 )
 		map_size += 2;
@@ -150,7 +141,7 @@ uint16_t	map_size;
 	// Create a generic netflow v1 extension map
 	v1_extension_info.map = (extension_map_t *)malloc((size_t)map_size);
 	if ( !v1_extension_info.map ) {
-		syslog(LOG_ERR, "Process_v1: malloc() error in %s line %d: %s\n", __FILE__, __LINE__, strerror (errno));
+		LogError("Process_v1: malloc() error in %s line %d: %s\n", __FILE__, __LINE__, strerror (errno));
 		return 0;
 	}
 
@@ -194,7 +185,7 @@ char ipstr[IP_STRING_LEN];
 	// nothing found
 	*e = (exporter_v1_t *)malloc(sizeof(exporter_v1_t));
 	if ( !(*e)) {
-		syslog(LOG_ERR, "Process_v1: malloc() error in %s line %d: %s\n", __FILE__, __LINE__, strerror (errno));
+		LogError("Process_v1: malloc() error in %s line %d: %s\n", __FILE__, __LINE__, strerror (errno));
 		return NULL;
 	}
 	memset((void *)(*e), 0, sizeof(exporter_v1_t));
@@ -213,7 +204,7 @@ char ipstr[IP_STRING_LEN];
 	// copy the v1 generic extension map
 	(*e)->extension_map		= (extension_map_t *)malloc(v1_extension_info.map->size);
 	if ( !(*e)->extension_map ) {
-		syslog(LOG_ERR, "Process_v1: malloc() error in %s line %d: %s\n", __FILE__, __LINE__, strerror (errno));
+		LogError("Process_v1: malloc() error in %s line %d: %s\n", __FILE__, __LINE__, strerror (errno));
 		free(*e);
 		*e = NULL;
 		return NULL;
@@ -245,7 +236,7 @@ char ipstr[IP_STRING_LEN];
 
 	dbg_printf("New Exporter: v1 SysID: %u, Extension ID: %i, IP: %s, \n", 
 		(*e)->info.sysid, (*e)->extension_map->map_id, ipstr);
-	syslog(LOG_INFO, "Process_v1: SysID: %u, New exporter: IP: %s\n", (*e)->info.sysid, ipstr);
+	LogError("Process_v1: SysID: %u, New exporter: IP: %s\n", (*e)->info.sysid, ipstr);
 
 	return (*e);
 
@@ -270,7 +261,7 @@ char		*string;
 
 		exporter = GetExporter(fs, v1_header);
 		if ( !exporter ) {
-			syslog(LOG_ERR,"Process_v1: Exporter NULL: Abort v1 record processing");
+			LogError("Process_v1: Exporter NULL: Abort v1 record processing");
 			return;
 		}
 		flags = 0;
@@ -293,14 +284,14 @@ char		*string;
 			// count check
 	  		count	= ntohs(v1_header->count);
 			if ( count > NETFLOW_V1_MAX_RECORDS ) {
-				syslog(LOG_ERR,"Process_v1: Unexpected record count in header: %i. Abort v1 record processing", count);
+				LogError("Process_v1: Unexpected record count in header: %i. Abort v1 record processing", count);
 				fs->nffile->buff_ptr = (void *)common_record;
 				return;
 			}
 
 			// input buffer size check for all expected records
 			if ( size_left < ( NETFLOW_V1_HEADER_LENGTH + count * flow_record_length) ) {
-				syslog(LOG_ERR,"Process_v1: Not enough data to process v1 record. Abort v1 record processing");
+				LogError("Process_v1: Not enough data to process v1 record. Abort v1 record processing");
 				fs->nffile->buff_ptr = (void *)common_record;
 				return;
 			}
@@ -308,7 +299,7 @@ char		*string;
 			// output buffer size check for all expected records
 			if ( !CheckBufferSpace(fs->nffile, count * v1_output_record_size) ) {
 				// fishy! - should never happen. maybe disk full?
-				syslog(LOG_ERR,"Process_v1: output buffer size error. Abort v1 record processing");
+				LogError("Process_v1: output buffer size error. Abort v1 record processing");
 				return;
 			}
 
@@ -385,7 +376,7 @@ char		*string;
 
 						default:
 							// this should never happen, as v1 has no other extensions
-							syslog(LOG_ERR,"Process_v1: Unexpected extension %i for v1 record. Skip extension", id);
+							LogError("Process_v1: Unexpected extension %i for v1 record. Skip extension", id);
 					}
 					j++;
 				}
@@ -503,9 +494,9 @@ char		*string;
 				// buffer size sanity check - should never happen, but check it anyway
 				bsize = (pointer_addr_t)common_record - (pointer_addr_t)fs->nffile->block_header - sizeof(data_block_header_t);
 				if ( bsize > BUFFSIZE ) {
-					syslog(LOG_ERR,"### Software error ###: %s line %d", __FILE__, __LINE__);
-					syslog(LOG_ERR,"Process_v1: Output buffer overflow! Flush buffer and skip records.");
-					syslog(LOG_ERR,"Buffer size: size: %u, bsize: %llu > %u", fs->nffile->block_header->size, (unsigned long long)bsize, BUFFSIZE);
+					LogError("### Software error ###: %s line %d", __FILE__, __LINE__);
+					LogError("Process_v1: Output buffer overflow! Flush buffer and skip records.");
+					LogError("Buffer size: size: %u, bsize: %llu > %u", fs->nffile->block_header->size, (unsigned long long)bsize, BUFFSIZE);
 					// reset buffer
 					fs->nffile->block_header->size 		= 0;
 					fs->nffile->block_header->NumRecords = 0;
