@@ -1,4 +1,6 @@
 /*
+ *  Copyright (c) 2016, Peter Haag
+ *  Copyright (c) 2014, Peter Haag
  *  Copyright (c) 2009, Peter Haag
  *  Copyright (c) 2004-2008, SWITCH - Teleinformatikdienste fuer Lehre und Forschung
  *  All rights reserved.
@@ -11,7 +13,7 @@
  *   * Redistributions in binary form must reproduce the above copyright notice, 
  *     this list of conditions and the following disclaimer in the documentation 
  *     and/or other materials provided with the distribution.
- *   * Neither the name of SWITCH nor the names of its contributors may be 
+ *   * Neither the name of the author nor the names of its contributors may be 
  *     used to endorse or promote products derived from this software without 
  *     specific prior written permission.
  *  
@@ -27,12 +29,7 @@
  *  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
  *  POSSIBILITY OF SUCH DAMAGE.
  *  
- *  $Author: haag $
- *
- *  $Id: nfcapd.c 51 2010-01-29 09:01:54Z haag $
- *
- *  $LastChangedRevision: 51 $
- *	
+ *  Author: peter
  *
  */
 
@@ -69,7 +66,6 @@
 #include <arpa/inet.h>
 #include <fcntl.h>
 #include <signal.h>
-#include <syslog.h>
 #include <sys/mman.h>
 #include <string.h>
 #include <dirent.h>
@@ -153,7 +149,7 @@ static void daemonize(void);
 static void SetPriv(char *userid, char *groupid );
 
 static void run(packet_function_t receive_packet, int socket, send_peer_t peer, 
-	time_t twin, time_t t_begin, int report_seq, int use_subdirs, int compress, int do_xstat);
+	time_t twin, time_t t_begin, int report_seq, int use_subdirs, char *time_extension, int compress, int do_xstat);
 
 /* Functions */
 static void usage(char *name) {
@@ -164,7 +160,7 @@ static void usage(char *name) {
 					"-w\t\tSync file rotation with next 5min (default) interval\n"
 					"-t interval\tset the interval to rotate nfcapd files\n"
 					"-b host\t\tbind socket to host/IP addr\n"
-					"-j mcastgroup\tJoin multicast group <mcastgroup>\n"
+					"-J mcastgroup\tJoin multicast group <mcastgroup>\n"
 					"-p portnum\tlisten on port portnum\n"
 					"-l basdir \tset the output directory. (no default) \n"
 					"-S subdir\tSub directory format. see nfcapd(1) for format\n"
@@ -175,7 +171,8 @@ static void usage(char *name) {
 					"-R IP[/port]\tRepeat incoming packets to IP address/port\n"
 					"-s rate\tset default sampling rate (default 1)\n"
 					"-x process\tlaunch process after a new file becomes available\n"
-					"-z\t\tCompress flows in output file.\n"
+					"-j\t\tBZ2 compress flows in output file.\n"
+					"-z\t\tLZO compress flows in output file.\n"
 					"-B bufflen\tSet socket buffer to bufflen bytes\n"
 					"-e\t\tExpire data at each cycle.\n"
 					"-D\t\tFork to background\n"
@@ -184,6 +181,7 @@ static void usage(char *name) {
 					"-4\t\tListen on IPv4 (default).\n"
 					"-6\t\tListen on IPv6.\n"
 					"-V\t\tPrint version and exit.\n"
+					"-Z\t\tAdd timezone offset to filenamet.\n"
 					, name);
 } // End of usage
 
@@ -195,7 +193,7 @@ pid_t ret;
 		return;
 
 	if ( launcher_alive ) {
-		syslog(LOG_INFO, "Signal launcher[%i] to terminate.", pid);
+		LogInfo("Signal launcher[%i] to terminate.", pid);
 		kill(pid, SIGTERM);
 
 		// wait for launcher to teminate
@@ -205,22 +203,22 @@ pid_t ret;
 			sleep(1);
 		}
 		if ( i >= LAUNCHER_TIMEOUT ) {
-			syslog(LOG_WARNING, "Laucher does not want to terminate - signal again");
+			LogError("Launcher does not want to terminate - signal again");
 			kill(pid, SIGTERM);
 			sleep(1);
 		}
 	} else {
-		syslog(LOG_ERR, "launcher[%i] already dead.", pid);
+		LogError("launcher[%i] already dead.", pid);
 	}
 
 	if ( (ret = waitpid (pid, &stat, 0)) == -1 ) {
-		syslog(LOG_ERR, "wait for launcher failed: %s %i", strerror(errno), ret);
+		LogError("wait for launcher failed: %s %i", strerror(errno), ret);
 	} else {
 		if ( WIFEXITED(stat) ) {
-			syslog(LOG_INFO, "launcher exit status: %i", WEXITSTATUS(stat));
+			LogInfo("launcher exit status: %i", WEXITSTATUS(stat));
 		}
 		if (  WIFSIGNALED(stat) ) {
-			syslog(LOG_WARNING, "launcher terminated due to signal %i", WTERMSIG(stat));
+			LogError("launcher terminated due to signal %i", WTERMSIG(stat));
 		}
 	}
 
@@ -314,7 +312,7 @@ int		err;
 	newuid = newgid = 0;
 	myuid = getuid();
 	if ( myuid != 0 ) {
-		syslog(LOG_ERR, "Only root wants to change uid/gid");
+		LogError("Only root wants to change uid/gid");
 		fprintf(stderr, "ERROR: Only root wants to change uid/gid\n");
 		exit(255);
 	}
@@ -340,7 +338,7 @@ int		err;
 
 		err = setgid(newgid);
 		if ( err ) {
-			syslog(LOG_ERR, "Can't set group id %ld for group '%s': %s",   (long)newgid, groupid, strerror(errno));
+			LogError("Can't set group id %ld for group '%s': %s",   (long)newgid, groupid, strerror(errno));
 			fprintf (stderr,"Can't set group id %ld for group '%s': %s\n", (long)newgid, groupid, strerror(errno));
 			exit(255);
 		}
@@ -350,7 +348,7 @@ int		err;
 	if ( newuid ) {
 		err = setuid(newuid);
 		if ( err ) {
-			syslog(LOG_ERR, "Can't set user id %ld for user '%s': %s",   (long)newuid, userid, strerror(errno));
+			LogError("Can't set user id %ld for user '%s': %s",   (long)newuid, userid, strerror(errno));
 			fprintf (stderr,"Can't set user id %ld for user '%s': %s\n", (long)newuid, userid, strerror(errno));
 			exit(255);
 		}
@@ -362,7 +360,7 @@ int		err;
 #include "collector_inline.c"
 
 static void run(packet_function_t receive_packet, int socket, send_peer_t peer, 
-	time_t twin, time_t t_begin, int report_seq, int use_subdirs, int compress, int do_xstat) {
+	time_t twin, time_t t_begin, int report_seq, int use_subdirs, char *time_extension, int compress, int do_xstat) {
 common_flow_header_t	*nf_header;
 FlowSource_t			*fs;
 struct sockaddr_storage nf_sender;
@@ -447,15 +445,15 @@ srecord_t	*commbuff;
 #endif
 
 			if ( cnt == -1 && errno != EINTR ) {
-				syslog(LOG_ERR, "ERROR: recvfrom: %s", strerror(errno));
+				LogError("ERROR: recvfrom: %s", strerror(errno));
 				continue;
 			}
 
 			if ( peer.hostname ) {
-				size_t len;
+				ssize_t len;
 				len = sendto(peer.sockfd, in_buff, cnt, 0, (struct sockaddr *)&(peer.addr), peer.addrlen);
 				if ( len < 0 ) {
-					syslog(LOG_ERR, "ERROR: sendto(): %s", strerror(errno));
+					LogError("ERROR: sendto(): %s", strerror(errno));
 				}
 			}
 		}
@@ -468,30 +466,28 @@ srecord_t	*commbuff;
 		if ( ((t_now - t_start) >= twin) || done ) {
 			char subfilename[64];
 			struct  tm *now;
-			char	*subdir;
+			char	*subdir, fmt[64];
 
 			alarm(0);
 			now = localtime(&t_start);
+			strftime(fmt, sizeof fmt, time_extension, now);
 
 			// prepare sub dir hierarchy
 			if ( use_subdirs ) {
 				subdir = GetSubDir(now);
 				if ( !subdir ) {
 					// failed to generate subdir path - put flows into base directory
-					syslog(LOG_ERR, "Failed to create subdir path!");
+					LogError("Failed to create subdir path!");
 			
 					// failed to generate subdir path - put flows into base directory
 					subdir = NULL;
-					snprintf(subfilename, 63, "nfcapd.%i%02i%02i%02i%02i",
-						now->tm_year + 1900, now->tm_mon + 1, now->tm_mday, now->tm_hour, now->tm_min);
+					snprintf(subfilename, 63, "nfcapd.%s", fmt);
 				} else {
-					snprintf(subfilename, 63, "%s/nfcapd.%i%02i%02i%02i%02i", subdir,
-						now->tm_year + 1900, now->tm_mon + 1, now->tm_mday, now->tm_hour, now->tm_min);
+					snprintf(subfilename, 63, "%s/nfcapd.%s", subdir, fmt);
 				}
 			} else {
 				subdir = NULL;
-				snprintf(subfilename, 63, "nfcapd.%i%02i%02i%02i%02i",
-					now->tm_year + 1900, now->tm_mon + 1, now->tm_mday, now->tm_hour, now->tm_min);
+				snprintf(subfilename, 63, "nfcapd.%s", fmt);
 			}
 			subfilename[63] = '\0';
 
@@ -511,7 +507,7 @@ srecord_t	*commbuff;
 				if ( nffile->block_header->NumRecords ) {
 					// flush current buffer to disc
 					if ( WriteBlock(nffile) <= 0 )
-						syslog(LOG_ERR, "Ident: %s, failed to write output buffer to disk: '%s'" , fs->Ident, strerror(errno));
+						LogError("Ident: %s, failed to write output buffer to disk: '%s'" , fs->Ident, strerror(errno));
 				} // else - no new records in current block
 
 	
@@ -533,7 +529,7 @@ srecord_t	*commbuff;
 
 				if ( fs->xstat ) {
 					if ( WriteExtraBlock(nffile, fs->xstat->block_header ) <= 0 ) 
-						syslog(LOG_ERR, "Ident: %s, failed to write xstat buffer to disk: '%s'" , fs->Ident, strerror(errno));
+						LogError("Ident: %s, failed to write xstat buffer to disk: '%s'" , fs->Ident, strerror(errno));
 
 					ResetPortHistogram(fs->xstat->port_histogram);
 					ResetBppHistogram(fs->xstat->bpp_histogram);
@@ -547,15 +543,15 @@ srecord_t	*commbuff;
 				if ( subdir && !SetupSubDir(fs->datadir, subdir, error, 255) ) {
 					// in this case the flows get lost! - the rename will fail
 					// but this should not happen anyway, unless i/o problems, inode problems etc.
-					syslog(LOG_ERR, "Ident: %s, Failed to create sub hier directories: %s", fs->Ident, error );
+					LogError("Ident: %s, Failed to create sub hier directories: %s", fs->Ident, error );
 				}
 
 				// if rename fails, we are in big trouble, as we need to get rid of the old .current file
 				// otherwise, we will loose flows and can not continue collecting new flows
 				err = rename(fs->current, nfcapd_filename);
 				if ( err ) {
-					syslog(LOG_ERR, "Ident: %s, Can't rename dump file: %s", fs->Ident,  strerror(errno));
-					syslog(LOG_ERR, "Ident: %s, Serious Problem! Fix manually", fs->Ident);
+					LogError("Ident: %s, Can't rename dump file: %s", fs->Ident,  strerror(errno));
+					LogError("Ident: %s, Serious Problem! Fix manually", fs->Ident);
 					if ( launcher_pid )
 						commbuff->failed = 1;
 
@@ -572,7 +568,7 @@ srecord_t	*commbuff;
 				}
 
 				// log stats
-				syslog(LOG_INFO,"Ident: '%s' Flows: %llu, Packets: %llu, Bytes: %llu, Sequence Errors: %u, Bad Packets: %u", 
+				LogInfo("Ident: '%s' Flows: %llu, Packets: %llu, Bytes: %llu, Sequence Errors: %u, Bad Packets: %u", 
 					fs->Ident, (unsigned long long)nffile->stat_record->numflows, (unsigned long long)nffile->stat_record->numpackets, 
 					(unsigned long long)nffile->stat_record->numbytes, nffile->stat_record->sequence_failure, fs->bad_packets);
 
@@ -617,14 +613,14 @@ srecord_t	*commbuff;
 					commbuff->subdir[0] = '\0';
 
 				if ( launcher_alive ) {
-					syslog(LOG_DEBUG, "Signal launcher");
+					LogInfo("Signal launcher");
 					kill(launcher_pid, SIGHUP);
 				} else 
-					syslog(LOG_ERR, "ERROR: Launcher did unexpectedly!");
+					LogError("ERROR: Launcher died unexpectedly!");
 
 			}
 			
-			syslog(LOG_INFO, "Total ignored packets: %u", ignored_packets);
+			LogInfo("Total ignored packets: %u", ignored_packets);
 			ignored_packets = 0;
 
 			if ( done )
@@ -653,7 +649,7 @@ srecord_t	*commbuff;
 				break;
 			else {
 				/* this should never be executed as it should be caught in other places */
-				syslog(LOG_ERR, "error condition in '%s', line '%d', cnt: %i", __FILE__, __LINE__ ,(int)cnt);
+				LogError("error condition in '%s', line '%d', cnt: %i", __FILE__, __LINE__ ,(int)cnt);
 				continue;
 			}
 		}
@@ -667,7 +663,7 @@ srecord_t	*commbuff;
 		if ( fs == NULL ) {
 			fs = AddDynamicSource(&FlowSource, &nf_sender);
 			if ( fs == NULL ) {
-				syslog(LOG_WARNING, "Skip UDP packet. Ignored packets so far %u packets", ignored_packets);
+				LogError("Skip UDP packet. Ignored packets so far %u packets", ignored_packets);
 				ignored_packets++;
 				continue;
 			}
@@ -679,13 +675,13 @@ srecord_t	*commbuff;
 			fs->nffile = OpenNewFile(fs->current, NULL, compress, 0, NULL);
 			if ( !fs->nffile ) {
 				LogError("Failed to open new collector file");
-				continue;
+				return;
 			}
 		}
 
 		/* check for too little data - cnt must be > 0 at this point */
 		if ( cnt < sizeof(common_flow_header_t) ) {
-			syslog(LOG_WARNING, "Ident: %s, Data length error: too little data for common netflow header. cnt: %i",fs->Ident, (int)cnt);
+			LogError("Ident: %s, Data length error: too little data for common netflow header. cnt: %i",fs->Ident, (int)cnt);
 			fs->bad_packets++;
 			continue;
 		}
@@ -712,7 +708,7 @@ srecord_t	*commbuff;
 				if ( verbose ) {
 					uint16_t count = ntohs(nf_header->count);
 					if ( blast_cnt != count ) {
-							// LogError("Missmatch blast check: Expected %u got %u\n", blast_cnt, count);
+							// LogError("Mismatch blast check: Expected %u got %u\n", blast_cnt, count);
 						blast_cnt = count;
 						blast_failures++;
 					} else {
@@ -726,7 +722,7 @@ srecord_t	*commbuff;
 				}
 			default:
 				// data error, while reading data from socket
-				syslog(LOG_ERR,"Ident: %s, Error reading netflow header: Unexpected netflow version %i", fs->Ident, version);
+				LogError("Ident: %s, Error reading netflow header: Unexpected netflow version %i", fs->Ident, version);
 				fs->bad_packets++;
 				continue;
 
@@ -743,7 +739,7 @@ srecord_t	*commbuff;
 			fs->nffile->block_header->size 		 = 0;
 			fs->nffile->block_header->NumRecords = 0;
 			fs->nffile->buff_ptr = (void *)((pointer_addr_t)fs->nffile->block_header + sizeof(data_block_header_t) );
-			syslog(LOG_ERR, "### Software bug ### Ident: %s, output buffer overflow: expect memory inconsitency", fs->Ident);
+			LogError("### Software bug ### Ident: %s, output buffer overflow: expect memory inconsitency", fs->Ident);
 		}
 	}
 
@@ -762,20 +758,24 @@ srecord_t	*commbuff;
 
 int main(int argc, char **argv) {
  
-char	*bindhost, *filter, *datadir, pidstr[32], *launch_process;
+char	*bindhost, *datadir, pidstr[32], *launch_process;
 char	*userid, *groupid, *checkptr, *listenport, *mcastgroup, *extension_tags;
-char	*Ident, *pcap_file, *dynsrcdir, pidfile[MAXPATHLEN];
+char	*Ident, *dynsrcdir, *time_extension, pidfile[MAXPATHLEN];
 struct stat fstat;
-srecord_t	*commbuff;
 packet_function_t receive_packet;
 send_peer_t  peer;
 FlowSource_t *fs;
 struct sigaction act;
 int		family, bufflen;
 time_t 	twin, t_start;
-int		sock, err, synctime, do_daemonize, expire, report_sequence, do_xstat;
+int		sock, synctime, do_daemonize, expire, spec_time_extension, report_sequence, do_xstat;
 int		subdir_index, sampling_rate, compress;
 int		c;
+#ifdef PCAP
+char	*pcap_file;
+ 
+	pcap_file		= NULL;
+#endif
 
 	receive_packet 	= recvfrom;
 	verbose = synctime = do_daemonize = 0;
@@ -788,25 +788,25 @@ int		c;
 	bindhost 		= NULL;
 	mcastgroup		= NULL;
 	pidfile[0]		= 0;
-	filter   		= NULL;
 	launch_process	= NULL;
 	userid 			= groupid = NULL;
 	twin	 		= TIME_WINDOW;
 	datadir	 		= NULL;
 	subdir_index	= 0;
+	time_extension	= "%Y%m%d%H%M";
+	spec_time_extension = 0;
 	expire			= 0;
 	sampling_rate	= 1;
-	compress		= 0;
+	compress		= NOT_COMPRESSED;
 	do_xstat		= 0;
 	memset((void *)&peer, 0, sizeof(send_peer_t));
 	peer.family		= AF_UNSPEC;
 	Ident			= "none";
 	FlowSource		= NULL;
 	extension_tags	= DefaultExtensions;
-	pcap_file		= NULL;
 	dynsrcdir		= NULL;
 
-	while ((c = getopt(argc, argv, "46ef:whEVI:DB:b:j:l:M:n:p:P:R:S:s:T:t:x:Xru:g:z")) != EOF) {
+	while ((c = getopt(argc, argv, "46ef:whEVI:DB:b:jl:J:M:n:p:P:R:S:s:T:t:x:Xru:g:zZ")) != EOF) {
 		switch (c) {
 			case 'h':
 				usage(argv[0]);
@@ -857,7 +857,10 @@ int		c;
 					fprintf(stderr, "ERROR: Path too long!\n");
 					exit(255);
 				}
-				err  = stat(dynsrcdir, &fstat);
+				if ( stat(dynsrcdir, &fstat) < 0 ) {
+					fprintf(stderr, "stat() failed on %s: %s\n", dynsrcdir, strerror(errno));
+					exit(255);
+				}
 				if ( !(fstat.st_mode & S_IFDIR) ) {
 					fprintf(stderr, "No such directory: %s\n", dynsrcdir);
 					break;
@@ -883,7 +886,7 @@ int		c;
 			case 'b':
 				bindhost = optarg;
 				break;
-			case 'j':
+			case 'J':
 				mcastgroup = optarg;
 				break;
 			case 'p':
@@ -942,7 +945,10 @@ int		c;
 					fprintf(stderr, "ERROR: Path too long!\n");
 					exit(255);
 				}
-				err  = stat(datadir, &fstat);
+				if ( stat(datadir, &fstat) < 0 ) {
+					fprintf(stderr, "stat() failed on %s: %s\n", datadir, strerror(errno));
+					exit(255);
+				}
 				if ( !(fstat.st_mode & S_IFDIR) ) {
 					fprintf(stderr, "No such directory: %s\n", datadir);
 					break;
@@ -964,8 +970,23 @@ int		c;
 			case 'x':
 				launch_process = optarg;
 				break;
+			case 'j':
+				if ( compress ) {
+					LogError("Use either -z for LZO or -j for BZ2 compression, but not both\n");
+					exit(255);
+				}
+				compress = BZ2_COMPRESSED;
+				break;
 			case 'z':
-				compress = 1;
+				if ( compress ) {
+					LogError("Use either -z for LZO or -j for BZ2 compression, but not both\n");
+					exit(255);
+				}
+				compress = LZO_COMPRESSED;
+				break;
+			case 'Z':
+				time_extension	= "%Y%m%d%H%M%z";
+				spec_time_extension = 1;
 				break;
 			case '4':
 				if ( family == AF_UNSPEC )
@@ -1003,11 +1024,16 @@ int		c;
 		exit(255);
 	}
 
-	if ( !InitLog(argv[0], SYSLOG_FACILITY)) {
+	if ( do_daemonize && !InitLog(argv[0], SYSLOG_FACILITY)) {
 		exit(255);
 	}
 
-	InitExtensionMaps(NULL);
+	if ( expire && spec_time_extension ) {
+		fprintf(stderr, "ERROR, -Z timezone extension breaks expire -e\n");
+		exit(255);
+	}
+
+	InitExtensionMaps(NO_EXTENSION_LIST);
 	SetupExtensionDescriptors(strdup(extension_tags));
 
 	// Debug code to read from pcap file
@@ -1034,7 +1060,7 @@ int		c;
 											&peer.addr, &peer.addrlen );
 		if ( peer.sockfd <= 0 )
 			exit(255);
-		syslog(LOG_DEBUG, "Replay flows to host: %s port: %s", peer.hostname, peer.port);
+		LogInfo("Replay flows to host: %s port: %s", peer.hostname, peer.port);
 	}
 
 	if ( sampling_rate < 0 ) {
@@ -1094,11 +1120,7 @@ int		c;
 		usage(argv[0]);
 		close(sock);
 		exit(255);
-	} else {
-		/* user specified a pcap filter */
-		filter = argv[optind];
-	}
-
+	} 
 
 	t_start = time(NULL);
 	if ( synctime )
@@ -1112,13 +1134,13 @@ int		c;
 		pid_t pid = getpid();
 		int pidf  = open(pidfile, O_RDWR|O_TRUNC|O_CREAT, S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH);
 		if ( pidf == -1 ) {
-			syslog(LOG_ERR, "Error opening pid file: '%s' %s", pidfile, strerror(errno));
+			LogError("Error opening pid file: '%s' %s", pidfile, strerror(errno));
 			close(sock);
 			exit(255);
 		}
 		snprintf(pidstr,31,"%lu\n", (unsigned long)pid);
 		if ( write(pidf, pidstr, strlen(pidstr)) <= 0 ) {
-			syslog(LOG_ERR, "Error write pid file: '%s' %s", pidfile, strerror(errno));
+			LogError("Error write pid file: '%s' %s", pidfile, strerror(errno));
 		}
 		close(pidf);
 	}
@@ -1132,13 +1154,11 @@ int		c;
 		// prepare shared memory
 		shmem = mmap(0, sizeof(srecord_t), PROT_READ | PROT_WRITE, MAP_ANON | MAP_SHARED, -1, 0);
 		if ( shmem == (caddr_t)-1 ) {
-			syslog(LOG_ERR, "mmap() error: %s", strerror(errno));
+			LogError("mmap() error: %s", strerror(errno));
 			close(sock);
 			exit(255);
 		}
 
-		commbuff = (srecord_t *)shmem;
-
 		launcher_pid = fork();
 		switch (launcher_pid) {
 			case 0:
@@ -1148,7 +1168,7 @@ int		c;
 				_exit(0);
 				break;
 			case -1:
-				syslog(LOG_ERR, "fork() error: %s", strerror(errno));
+				LogError("fork() error: %s", strerror(errno));
 				if ( strlen(pidfile) )
 					unlink(pidfile);
 				exit(255);
@@ -1156,14 +1176,14 @@ int		c;
 			default:
 				// parent
 			launcher_alive = 1;
-			syslog(LOG_DEBUG, "Launcher[%i] forked", launcher_pid);
+			LogInfo("Launcher[%i] forked", launcher_pid);
 		}
 	}
 
 	fs = FlowSource;
 	while ( fs ) {
 		if ( InitBookkeeper(&fs->bookkeeper, fs->datadir, getpid(), launcher_pid) != BOOKKEEPER_OK ) {
-			syslog(LOG_ERR, "initialize bookkeeper failed.");
+			LogError("initialize bookkeeper failed.");
 
 			// release all already allocated bookkeepers
 			fs = FlowSource;
@@ -1199,8 +1219,9 @@ int		c;
 	sigaction(SIGALRM, &act, NULL);
 	sigaction(SIGCHLD, &act, NULL);
 
-	syslog(LOG_INFO, "Startup.");
-	run(receive_packet, sock, peer, twin, t_start, report_sequence, subdir_index, compress, do_xstat);
+	LogInfo("Startup.");
+	run(receive_packet, sock, peer, twin, t_start, report_sequence, subdir_index, 
+		time_extension, compress, do_xstat);
 	close(sock);
 	kill_launcher(launcher_pid);
 
@@ -1211,15 +1232,15 @@ int		c;
 		if ( expire == 0 && ReadStatInfo(fs->datadir, &dirstat, LOCK_IF_EXISTS) == STATFILE_OK ) {
 			UpdateBookStat(dirstat, fs->bookkeeper);
 			WriteStatInfo(dirstat);
-			syslog(LOG_INFO, "Updating statinfo in directory '%s'", datadir);
+			LogInfo("Updating statinfo in directory '%s'", datadir);
 		}
 
 		ReleaseBookkeeper(fs->bookkeeper, DESTROY_BOOKKEEPER);
 		fs = fs->next;
 	}
 
-	syslog(LOG_INFO, "Terminating nfcapd.");
-	closelog();
+	LogInfo("Terminating nfcapd.");
+	EndLog();
 
 	if ( strlen(pidfile) )
 		unlink(pidfile);
