@@ -1,4 +1,5 @@
 /*
+ *  Copyright (c) 2014, Peter Haag
  *  Copyright (c) 2009, Peter Haag
  *  Copyright (c) 2004-2008, SWITCH - Teleinformatikdienste fuer Lehre und Forschung
  *  All rights reserved.
@@ -11,7 +12,7 @@
  *   * Redistributions in binary form must reproduce the above copyright notice, 
  *	 this list of conditions and the following disclaimer in the documentation 
  *	 and/or other materials provided with the distribution.
- *   * Neither the name of SWITCH nor the names of its contributors may be 
+ *   * Neither the name of the author nor the names of its contributors may be 
  *	 used to endorse or promote products derived from this software without 
  *	 specific prior written permission.
  *  
@@ -48,28 +49,38 @@ static inline void UpdateStat(stat_record_t	*stat_record, master_record_t *maste
 	switch (master_record->prot) {
 		case IPPROTO_ICMP:
 		case IPPROTO_ICMPV6:
-			stat_record->numflows_icmp++;
+			stat_record->numflows_icmp   += master_record->aggr_flows ? master_record->aggr_flows : 1;
 			stat_record->numpackets_icmp += master_record->dPkts;
+			stat_record->numpackets_icmp += master_record->out_pkts;
 			stat_record->numbytes_icmp   += master_record->dOctets;
+			stat_record->numbytes_icmp   += master_record->out_bytes;
 			break;
 		case IPPROTO_TCP:
-			stat_record->numflows_tcp++;
+			stat_record->numflows_tcp   += master_record->aggr_flows ? master_record->aggr_flows : 1;
 			stat_record->numpackets_tcp += master_record->dPkts;
+			stat_record->numpackets_tcp += master_record->out_pkts;
 			stat_record->numbytes_tcp   += master_record->dOctets;
+			stat_record->numbytes_tcp   += master_record->out_bytes;
 			break;
 		case IPPROTO_UDP:
-			stat_record->numflows_udp++;
+			stat_record->numflows_udp   += master_record->aggr_flows ? master_record->aggr_flows : 1;
 			stat_record->numpackets_udp += master_record->dPkts;
+			stat_record->numpackets_udp += master_record->out_pkts;
 			stat_record->numbytes_udp   += master_record->dOctets;
+			stat_record->numbytes_udp   += master_record->out_bytes;
 			break;
 		default:
-			stat_record->numflows_other++;
+			stat_record->numflows_other   += master_record->aggr_flows ? master_record->aggr_flows : 1;
 			stat_record->numpackets_other += master_record->dPkts;
+			stat_record->numpackets_other += master_record->out_pkts;
 			stat_record->numbytes_other   += master_record->dOctets;
+			stat_record->numbytes_other   += master_record->out_bytes;
 	}
-	stat_record->numflows++;
+	stat_record->numflows   += master_record->aggr_flows ? master_record->aggr_flows : 1;
 	stat_record->numpackets	+= master_record->dPkts;
+	stat_record->numpackets	+= master_record->out_pkts;
 	stat_record->numbytes 	+= master_record->dOctets;
+	stat_record->numbytes 	+= master_record->out_bytes;
 
 	if ( master_record->first < stat_record->first_seen ) {
 		stat_record->first_seen = master_record->first;
