@@ -1,4 +1,5 @@
 /*
+ *  Copyright (c) 2014, Peter Haag
  *  Copyright (c) 2009, Peter Haag
  *  Copyright (c) 2004-2008, SWITCH - Teleinformatikdienste fuer Lehre und Forschung
  *  All rights reserved.
@@ -11,7 +12,7 @@
  *   * Redistributions in binary form must reproduce the above copyright notice, 
  *     this list of conditions and the following disclaimer in the documentation 
  *     and/or other materials provided with the distribution.
- *   * Neither the name of SWITCH nor the names of its contributors may be 
+ *   * Neither the name of the author nor the names of its contributors may be 
  *     used to endorse or promote products derived from this software without 
  *     specific prior written permission.
  *  
@@ -176,7 +177,7 @@ int i;
 		(*c)->datadir = dirlist.list[i];
 		(*c)->do_rescan = do_rescan;
 
-		ret = ReadStatInfo((*c)->datadir, &(*c)->dirstat, CRETAE_AND_LOCK);
+		ret = ReadStatInfo((*c)->datadir, &(*c)->dirstat, CREATE_AND_LOCK);
 		switch (ret) {
 			case FORCE_REBUILD:
 				printf("Force rebuild requested by stat record in %s\n", (*c)->datadir);
@@ -213,6 +214,7 @@ int 		c, err, maxsize_set, maxlife_set;
 int			do_rescan, do_expire, do_list, print_stat, do_update_param, print_books, is_profile, nfsen_format;
 char		*maxsize_string, *lifetime_string, *datadir;
 uint64_t	maxsize, lifetime, low_water;
+uint32_t	runtime;
 channel_t	*channel, *current_channel;
 
 	maxsize_string = lifetime_string = NULL;
@@ -229,8 +231,9 @@ channel_t	*channel, *current_channel;
 	maxlife_set 	= 0;
 	low_water		= 0;
 	nfsen_format	= 0;
+	runtime			= 0;
 
-	while ((c = getopt(argc, argv, "e:hl:L:Ypr:s:t:u:w:")) != EOF) {
+	while ((c = getopt(argc, argv, "e:hl:L:T:Ypr:s:t:u:w:")) != EOF) {
 		switch (c) {
 			case 'h':
 				usage(argv[0]);
@@ -287,6 +290,13 @@ channel_t	*channel, *current_channel;
 				if ( low_water == 0 )
 					low_water = 100;
 				break;
+			case 'T':
+				runtime = strtoll(optarg, NULL, 10);
+				if ( runtime > 3600 ) {
+					fprintf(stderr, "Runtime > 3600 (1h)\n");
+					exit(250);
+				}
+				break;
 			case 'Y':
 				nfsen_format = 1;
 				break;
@@ -366,7 +376,7 @@ channel_t	*channel, *current_channel;
 				printf("Rescan again, due to file changes in directory!\n");
 			}
 			if ( BookSequence(current_channel->books) != last_sequence ) {
-				fprintf(stderr, "Could not savely rescan the directory. Data is not consistent.\n");
+				fprintf(stderr, "Could not safely rescan the directory. Data is not consistent.\n");
 				ReleaseBookkeeper(current_channel->books, DETACH_ONLY);
 				if ( current_channel->status == OK )
 					WriteStatInfo(current_channel->dirstat);
@@ -409,7 +419,7 @@ channel_t	*channel, *current_channel;
 				current_channel = current_channel->next;
 			}
 			old_stat = current_stat;
-			ExpireProfile(channel, &current_stat, maxsize, lifetime);
+			ExpireProfile(channel, &current_stat, maxsize, lifetime, runtime);
 
 		} else {
 			// cmd args override dirstat values
@@ -424,7 +434,7 @@ channel_t	*channel, *current_channel;
 	
 		
 			old_stat = *(channel->dirstat);
-			ExpireDir(channel->datadir, channel->dirstat, maxsize, lifetime);
+			ExpireDir(channel->datadir, channel->dirstat, maxsize, lifetime, runtime);
 			current_stat = *(channel->dirstat);
 
 		}
