@@ -1,4 +1,5 @@
 /*
+ *  Copyright (c) 2014, Peter Haag
  *  Copyright (c) 2009, Peter Haag
  *  Copyright (c) 2004-2008, SWITCH - Teleinformatikdienste fuer Lehre und Forschung
  *  All rights reserved.
@@ -11,7 +12,7 @@
  *   * Redistributions in binary form must reproduce the above copyright notice, 
  *	 this list of conditions and the following disclaimer in the documentation 
  *	 and/or other materials provided with the distribution.
- *   * Neither the name of SWITCH nor the names of its contributors may be 
+ *   * Neither the name of the author nor the names of its contributors may be 
  *	 used to endorse or promote products derived from this software without 
  *	 specific prior written permission.
  *  
@@ -74,7 +75,9 @@ uint32_t msecs   = 10;
 
 static extension_info_t extension_info;
 
+#define NEED_PACKRECORD 1
 #include "nffile_inline.c"
+#undef NEED_PACKRECORD
 
 void *GenRecord(int af, void *buff_ptr, char *src_ip, char *dst_ip, int src_port, int dst_port, 
 	int proto, int tcp_flags, int tos, uint64_t packets, uint64_t bytes, int src_as, int dst_as);
@@ -215,10 +218,10 @@ nffile_t			*nffile;
 	extension_info.map->ex_id[i] = 0;
 	extension_info.map->size = sizeof(extension_map_t) + i * sizeof(uint16_t);
 
-    // align 32bits
-    if (( extension_info.map->size & 0x3 ) != 0 ) {
-        extension_info.map->size += 4 - ( extension_info.map->size & 0x3 );
-    }
+	// align 32bits
+	if (( extension_info.map->size & 0x3 ) != 0 ) {
+		extension_info.map->size += 4 - ( extension_info.map->size & 0x3 );
+	}
 
 	extension_info.map->extension_size = 0;
 	i=0;
@@ -229,7 +232,7 @@ nffile_t			*nffile;
 	}
 	memset((void *)&record, 0, sizeof(record));
 
-	nffile = OpenNewFile("-", NULL, 0, 0, NULL);
+	nffile = OpenNewFile("-", NULL, NOT_COMPRESSED, 0, NULL);
 	if ( !nffile ) {
 		exit(255);
 	}
