@@ -1,4 +1,6 @@
 /*
+ *  Copyright (c) 2017, Peter Haag
+ *  Copyright (c) 2016, Peter Haag
  *  Copyright (c) 2014, Peter Haag
  *  Copyright (c) 2013, Peter Haag
  *  All rights reserved.
@@ -27,16 +29,8 @@
  *  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
  *  POSSIBILITY OF SUCH DAMAGE.
  *  
- *  $Author$
- *
- *  $Id$
- *
- *  $LastChangedRevision$
- *  
  */
 
-/* $Id: pcapd.c 2778 2012-03-19 09:23:26Z roethlis $ */
-
 #include "config.h"
 
 #ifdef HAVE_FEATURES_H
@@ -62,6 +56,7 @@
 #include <errno.h>
 #include <signal.h>
 #include <string.h>
+#include <assert.h>
 
 #ifdef HAVE_STDINT_H
 #include <stdint.h>
@@ -81,7 +76,7 @@
 # include <net/bpf.h>
 #else 
 #ifdef HAVE_PCAP_BPF_H
-# include <pcap-bpf.h>
+# include <pcap/bpf.h>
 #else
 # error missing bpf header
 #endif 
@@ -100,6 +95,8 @@
 #include "nfxstat.h"
 #include "collector.h"
 #include "exporter.h"
+#include "rbtree.h"
+#include "ipfrag.h"
 
 #ifdef HAVE_FTS_H
 #   include <fts.h>
@@ -249,13 +246,14 @@ static void usage(char *name) {
 					"-i device\tspecify a device\n"
 					"-r pcapfile\tspecify a file to read from\n"
 					"-B cache buckets\tset the number of cache buckets. (default 1048576)\n"
-					"-s snaplen\tset the snapshot length\n"
+					"-s snaplen\tset the snapshot length - default 1500\n"
 					"-l flowdir \tset the flow output directory. (no default) \n"
-					"-l pcapdir \tset the pcapdir directory. (optional) \n"
+					"-p pcapdir \tset the pcapdir directory. (optional) \n"
 					"-S subdir\tSub directory format. see nfcapd(1) for format\n"
 					"-I Ident\tset the ident string for stat file. (default 'none')\n"
 					"-P pidfile\tset the PID file\n"
 					"-t time frame\tset the time window to rotate pcap/nfcapd file\n"
+					"-j\t\tBZ2 compress flows in output file.\n"
 					"-z\t\tCompress flows in output file.\n"
 					"-E\t\tPrint extended format of netflow data. for debugging purpose only.\n"
 					"-T\t\tInclude extension tags in records.\n"
@@ -612,11 +610,11 @@ __attribute__((noreturn)) static void *p_flow_thread(void *thread_data) {
 p_flow_thread_args_t *args = (p_flow_thread_args_t *)thread_data;
 time_t t_win				 = args->t_win;
 int subdir_index			 = args->subdir_index;
-int compress				 = args->compress;
+int compress			 	 = args->compress;
 FlowSource_t *fs			 = args->fs;
 
 // locals
-time_t t_start, t_clock, t_udp_flush;
+time_t t_start, t_clock;
 int err, done;
 
 	done 	   = 0;
@@ -657,7 +655,6 @@ int err, done;
 
 	t_start = 0;
 	t_clock = 0;
-	t_udp_flush = 0;
 	while ( 1 ) {
 		struct FlowNode	*Node;
 
@@ -671,7 +668,7 @@ int err, done;
 		}
 
 		if ( t_start == 0 ) {
-			t_udp_flush = t_start = t_clock - (t_clock % t_win);
+			t_start = t_clock - (t_clock % t_win);
 		}
 
 		if (((t_clock - t_start) >= t_win) || done) { /* rotate file */
@@ -794,11 +791,6 @@ int err, done;
 			}
 		}
 
-		if (((t_clock - t_udp_flush) >= 10) || !done) { /* flush inactive UDP list */
-			UDPexpire(fs, t_clock - 10 );
-			t_udp_flush = t_clock;
-		}
-
 		if ( Node->fin != SIGNAL_NODE )
 			// Process the Node
 			ProcessFlowNode(fs, Node);
@@ -1194,7 +1186,7 @@ pcap_dev_t 		*pcap_dev;
 p_packet_thread_args_t *p_packet_thread_args;
 p_flow_thread_args_t *p_flow_thread_args;
 
-	snaplen			= 100;
+	snaplen			= 1500;
 	do_daemonize	= 0;
 	launcher_pid	= 0;
 	device			= NULL;
@@ -1209,11 +1201,11 @@ p_flow_thread_args_t *p_flow_thread_args;
 	fs				= NULL;
 	extension_tags	= DefaultExtensions;
 	subdir_index	= 0;
-	compress		= 0;
+	compress		= NOT_COMPRESSED;
 	verbose			= 0;
 	expire			= 0;
 	cache_size		= 0;
-	while ((c = getopt(argc, argv, "B:DEg:hi:r:s:l:p:P:t:u:S:T:e:Vz")) != EOF) {
+	while ((c = getopt(argc, argv, "B:DEI:g:hi:j:r:s:l:p:P:t:u:S:T:e:Vz")) != EOF) {
 		switch (c) {
 			struct stat fstat;
 			case 'h':
@@ -1236,6 +1228,9 @@ p_flow_thread_args_t *p_flow_thread_args;
 					exit(EXIT_FAILURE);
 				}
 				break;
+			case 'I':
+				Ident = strdup(optarg);
+				break;
 			case 'i':
 				device = optarg;
 				break;
@@ -1284,8 +1279,19 @@ p_flow_thread_args_t *p_flow_thread_args;
 					exit(EXIT_FAILURE);
 				}
 				break;
+			case 'j':
+				if ( compress ) {
+					LogError("Use either -z for LZO or -j for BZ2 compression, but not both\n");
+					exit(255);
+				}
+				compress = BZ2_COMPRESSED;
+				break;
 			case 'z':
-				compress = 1;
+				if ( compress ) {
+					LogError("Use either -z for LZO or -j for BZ2 compression, but not both\n");
+					exit(255);
+				}
+				compress = LZO_COMPRESSED;
 				break;
 			case 'P':
 				if ( optarg[0] == '/' ) { 	// absolute path given
@@ -1450,7 +1456,6 @@ p_flow_thread_args_t *p_flow_thread_args;
 
 	// Init the extension map list
 	if ( !InitExtensionMapList(fs) ) {
-		// error message goes to syslog
 		pcap_close(pcap_dev->handle);
 		exit(255);
 	}
