@@ -235,7 +235,6 @@ int	v1_map_done = 0;
 		flow_record = nffile->buff_ptr;
 		for ( i=0; i < nffile->block_header->NumRecords; i++ ) {
 			switch ( flow_record->type ) { 
-					case CommonRecordV0Type:
 					case CommonRecordType: {
 					generic_exporter_t *exp_info = exporter_list[flow_record->exporter_sysid];
 					uint32_t map_id = flow_record->ext_map;
@@ -364,7 +363,6 @@ profile_param_info_t **list = &profile_list;
 
 	profile_list = NULL;
 	while ( ( fgets(line, 512, stdin) != NULL )) {
-		LogInfo("Process line '%s'\n", line);
 		line[511] = '\0';
 
 		if ( *list == NULL ) 
@@ -388,6 +386,7 @@ profile_param_info_t **list = &profile_list;
 		// <profilegroup>#<profilename>#<profiletype>#<channelname>#<channel_sourcelist>
 		p = strchr(line, '\n');
 		if ( p ) *p = '\0';
+		LogInfo("Process line '%s'\n", line);
 
 		q = line;
 		p = strchr(q, '#');
@@ -528,19 +527,18 @@ int main( int argc, char **argv ) {
 unsigned int		num_channels, compress;
 struct stat stat_buf;
 profile_param_info_t *profile_list;
-char *rfile, *ffile, *filename, *Mdirs, *tstring;
+char *rfile, *ffile, *filename, *Mdirs;
 char	*profile_datadir, *profile_statdir, *nameserver;
 int c, syntax_only, subdir_index, stdin_profile_params, do_xstat;
 time_t tslot;
 
-	tstring 		= NULL;
 	profile_datadir = NULL;
 	profile_statdir = NULL;
 	Mdirs 			= NULL;
 	tslot 			= 0;
 	syntax_only	    = 0;
 	do_xstat	    = 0;
-	compress		= 0;
+	compress		= NOT_COMPRESSED;
 	subdir_index	= 0;
 	profile_list	= NULL;
 	nameserver		= NULL;
@@ -553,7 +551,7 @@ time_t tslot;
 	// default file names
 	ffile = "filter.txt";
 	rfile = NULL;
-	while ((c = getopt(argc, argv, "D:HIL:p:P:hf:r:n:M:S:t:VzZ")) != EOF) {
+	while ((c = getopt(argc, argv, "D:HIL:p:P:hf:J;r:n:M:S:t:VzZ")) != EOF) {
 		switch (c) {
 			case 'h':
 				usage(argv[0]);
@@ -603,8 +601,19 @@ time_t tslot;
 			case 'r':
 				rfile = optarg;
 				break;
+			case 'j':
+				if ( compress ) {
+					LogError("Use either -z for LZO or -j for BZ2 compression, but not both\n");
+					exit(255);
+				}
+				compress = BZ2_COMPRESSED;
+				break;
 			case 'z':
-				compress = 1;
+				if ( compress ) {
+					LogError("Use either -z for LZO or -j for BZ2 compression, but not both\n");
+					exit(255);
+				}
+				compress = LZO_COMPRESSED;
 				break;
 			default:
 				usage(argv[0]);
