@@ -1,4 +1,6 @@
 /*
+ *  Copyright (c) 2016, Peter Haag
+ *  Copyright (c) 2014, Peter Haag
  *  Copyright (c) 2009, Peter Haag
  *  Copyright (c) 2004-2008, SWITCH - Teleinformatikdienste fuer Lehre und Forschung
  *  All rights reserved.
@@ -11,7 +13,7 @@
  *   * Redistributions in binary form must reproduce the above copyright notice, 
  *     this list of conditions and the following disclaimer in the documentation 
  *     and/or other materials provided with the distribution.
- *   * Neither the name of SWITCH nor the names of its contributors may be 
+ *   * Neither the name of the author nor the names of its contributors may be 
  *     used to endorse or promote products derived from this software without 
  *     specific prior written permission.
  *  
@@ -27,16 +29,11 @@
  *  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
  *  POSSIBILITY OF SUCH DAMAGE.
  *  
- *  $Author: haag $
- *
- *  $Id: sfcapd.c 69 2010-09-09 07:17:43Z haag $
- *
- *  $LastChangedRevision: 69 $
- *	
- *
  */
 
+#ifdef HAVE_CONFIG_H 
 #include "config.h"
+#endif
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -52,12 +49,12 @@
 #include <sys/wait.h>
 #include <sys/stat.h>
 #include <sys/param.h>
+#include <sys/time.h>
 #include <netinet/in.h>
 #include <arpa/inet.h>
 #include <time.h>
 #include <fcntl.h>
 #include <signal.h>
-#include <syslog.h>
 #include <sys/mman.h>
 #include <string.h>
 #include <dirent.h>
@@ -127,7 +124,8 @@ static void daemonize(void);
 
 static void SetPriv(char *userid, char *groupid );
 
-static void run(packet_function_t receive_packet, int socket, send_peer_t peer, time_t twin, time_t t_begin, int report_seq, char *datadir, int use_subdirs, int compress, int do_xstat);
+static void run(packet_function_t receive_packet, int socket, send_peer_t peer, 
+	time_t twin, time_t t_begin, int report_seq, int use_subdirs, char *time_extension, int compress, int do_xstat);
 
 /* Functions */
 static void usage(char *name) {
@@ -138,7 +136,7 @@ static void usage(char *name) {
 					"-w\t\tSync file rotation with next 5min (default) interval\n"
 					"-t interval\tset the interval to rotate sfcapd files\n"
 					"-b host\t\tbind socket to host/IP addr\n"
-					"-j mcastgroup\tJoin multicast group <mcastgroup>\n"
+					"-J mcastgroup\tJoin multicast group <mcastgroup>\n"
 					"-p portnum\tlisten on port portnum\n"
 					"-l logdir \tset the output directory. (no default) \n"
 					"-S subdir\tSub directory format. see nfcapd(1) for format\n"
@@ -148,7 +146,8 @@ static void usage(char *name) {
 					"-P pidfile\tset the PID file\n"
 					"-R IP[/port]\tRepeat incoming packets to IP address/port\n"
 					"-x process\tlaunch process after a new file becomes available\n"
-					"-z\t\tCompress flows in output file.\n"
+					"-j\t\tBZ2 compress flows in output file.\n"
+					"-z\t\tLZO compress flows in output file.\n"
 					"-B bufflen\tSet socket buffer to bufflen bytes\n"
 					"-e\t\tExpire data at each cycle.\n"
 					"-D\t\tFork to background\n"
@@ -156,6 +155,7 @@ static void usage(char *name) {
 					"-4\t\tListen on IPv4 (default).\n"
 					"-6\t\tListen on IPv6.\n"
 					"-V\t\tPrint version and exit.\n"
+					"-Z\t\tAdd timezone offset to filenamet.\n"
 					, name);
 } // End of usage
 
@@ -167,7 +167,7 @@ pid_t ret;
 		return;
 
 	if ( launcher_alive ) {
-		syslog(LOG_INFO, "Signal launcher[%i] to terminate.", pid);
+		LogInfo("Signal launcher[%i] to terminate.", pid);
 		kill(pid, SIGTERM);
 
 		// wait for launcher to teminate
@@ -177,22 +177,22 @@ pid_t ret;
 			sleep(1);
 		}
 		if ( i >= LAUNCHER_TIMEOUT ) {
-			syslog(LOG_WARNING, "Laucher does not want to terminate - signal again");
+			LogError("Launcher does not want to terminate - signal again");
 			kill(pid, SIGTERM);
 			sleep(1);
 		}
 	} else {
-		syslog(LOG_ERR, "launcher[%i] already dead.", pid);
+		LogError("launcher[%i] already dead.", pid);
 	}
 
 	if ( (ret = waitpid (pid, &stat, 0)) == -1 ) {
-		syslog(LOG_ERR, "wait for launcher failed: %s %i", strerror(errno), ret);
+		LogError("wait for launcher failed: %s %i", strerror(errno), ret);
 	} else {
 		if ( WIFEXITED(stat) ) {
-			syslog(LOG_INFO, "launcher exit status: %i", WEXITSTATUS(stat));
+			LogInfo("launcher exit status: %i", WEXITSTATUS(stat));
 		}
 		if (  WIFSIGNALED(stat) ) {
-			syslog(LOG_WARNING, "launcher terminated due to signal %i", WTERMSIG(stat));
+			LogError("launcher terminated due to signal %i", WTERMSIG(stat));
 		}
 	}
 
@@ -284,7 +284,7 @@ int		err;
 	newuid = newgid = 0;
 	myuid = getuid();
 	if ( myuid != 0 ) {
-		syslog(LOG_ERR, "Only root wants to change uid/gid");
+		LogError("Only root wants to change uid/gid");
 		fprintf(stderr, "ERROR: Only root wants to change uid/gid\n");
 		exit(255);
 	}
@@ -310,7 +310,7 @@ int		err;
 
 		err = setgid(newgid);
 		if ( err ) {
-			syslog(LOG_ERR, "Can't set group id %ld for group '%s': %s",   (long)newgid, groupid, strerror(errno));
+			LogError("Can't set group id %ld for group '%s': %s",   (long)newgid, groupid, strerror(errno));
 			fprintf (stderr,"Can't set group id %ld for group '%s': %s\n", (long)newgid, groupid, strerror(errno));
 			exit(255);
 		}
@@ -320,7 +320,7 @@ int		err;
 	if ( newuid ) {
 		err = setuid(newuid);
 		if ( err ) {
-			syslog(LOG_ERR, "Can't set user id %ld for user '%s': %s",   (long)newuid, userid, strerror(errno));
+			LogError("Can't set user id %ld for user '%s': %s",   (long)newuid, userid, strerror(errno));
 			fprintf (stderr,"Can't set user id %ld for user '%s': %s\n", (long)newuid, userid, strerror(errno));
 			exit(255);
 		}
@@ -331,7 +331,8 @@ int		err;
 #include "nffile_inline.c"
 #include "collector_inline.c"
 
-static void run(packet_function_t receive_packet, int socket, send_peer_t peer, time_t twin, time_t t_begin, int report_seq, char *datadir, int use_subdirs, int compress, int do_xstat) {
+static void run(packet_function_t receive_packet, int socket, send_peer_t peer, 
+	time_t twin, time_t t_begin, int report_seq, int use_subdirs, char *time_extension, int compress, int do_xstat) {
 FlowSource_t			*fs;
 struct sockaddr_storage sf_sender;
 socklen_t 	sf_sender_size = sizeof(sf_sender);
@@ -348,12 +349,13 @@ srecord_t	*commbuff;
 
 	in_buff  = malloc(NETWORK_INPUT_BUFF_SIZE);
 	if ( !in_buff ) {
-		syslog(LOG_ERR, "malloc() buffer allocation error: %s", strerror(errno));
+		LogError("malloc() buffer allocation error: %s", strerror(errno));
 		return;
 	}
 
 	// init vars
 	commbuff = (srecord_t *)shmem;
+	string = NULL;
 
 	// Init each netflow source output data buffer
 	fs = FlowSource;
@@ -362,7 +364,6 @@ srecord_t	*commbuff;
 		// prepare file
 		fs->nffile = OpenNewFile(fs->current, NULL, compress, 0, NULL);
 		if ( !fs->nffile ) {
-			syslog(LOG_ERR, "%s", string);
 			return;
 		}
 		if ( do_xstat ) {
@@ -414,15 +415,15 @@ srecord_t	*commbuff;
 				(struct sockaddr *)&sf_sender, &sf_sender_size);
 #endif
 			if ( cnt == -1 && errno != EINTR ) {
-				syslog(LOG_ERR, "ERROR: recvfrom: %s", strerror(errno));
+				LogError("ERROR: recvfrom: %s", strerror(errno));
 				continue;
 			}
 
 			if ( peer.hostname ) {
-				size_t len;
+				ssize_t len;
 				len = sendto(peer.sockfd, in_buff, cnt, 0, (struct sockaddr *)&(peer.addr), peer.addrlen);
 				if ( len < 0 ) {
-					syslog(LOG_ERR, "ERROR: sendto(): %s", strerror(errno));
+					LogError("ERROR: sendto(): %s", strerror(errno));
 				}
 			}
 		}
@@ -434,29 +435,27 @@ srecord_t	*commbuff;
 		if ( ((t_now - t_start) >= twin) || done ) {
 			char subfilename[64];
 			struct  tm *now;
-			char	*subdir;
+			char	*subdir, fmt[64];
 			alarm(0);
 			now = localtime(&t_start);
+			strftime(fmt, sizeof fmt, time_extension, now);
 
 			// prepare sub dir hierarchy
 			if ( use_subdirs ) {
 				subdir = GetSubDir(now);
 				if ( !subdir ) {
 					// failed to generate subdir path - put flows into base directory
-					syslog(LOG_ERR, "Failed to create subdir path!");
+					LogError("Failed to create subdir path!");
 			
 					// failed to generate subdir path - put flows into base directory
 					subdir = NULL;
-					snprintf(subfilename, 63, "nfcapd.%i%02i%02i%02i%02i",
-						now->tm_year + 1900, now->tm_mon + 1, now->tm_mday, now->tm_hour, now->tm_min);
+					snprintf(subfilename, 63, "nfcapd.%s", fmt);
 				} else {
-					snprintf(subfilename, 63, "%s/nfcapd.%i%02i%02i%02i%02i", subdir,
-						now->tm_year + 1900, now->tm_mon + 1, now->tm_mday, now->tm_hour, now->tm_min);
+					snprintf(subfilename, 63, "%s/nfcapd.%s", subdir, fmt);
 				}
 			} else {
 				subdir = NULL;
-				snprintf(subfilename, 63, "nfcapd.%i%02i%02i%02i%02i",
-					now->tm_year + 1900, now->tm_mon + 1, now->tm_mday, now->tm_hour, now->tm_min);
+				snprintf(subfilename, 63, "nfcapd.%s", fmt);
 			}
 			subfilename[63] = '\0';
 
@@ -476,7 +475,7 @@ srecord_t	*commbuff;
 				if ( nffile->block_header->NumRecords ) {
 					// flush current buffer to disc
 					if ( WriteBlock(nffile) <= 0 )
-						syslog(LOG_ERR, "Ident: %s, failed to write output buffer to disk: '%s'" , fs->Ident, strerror(errno));
+						LogError("Ident: %s, failed to write output buffer to disk: '%s'" , fs->Ident, strerror(errno));
 				} // else - no new records in current block
 
 	
@@ -492,7 +491,7 @@ srecord_t	*commbuff;
 	
 				if ( fs->xstat ) {
 					if ( WriteExtraBlock(nffile, fs->xstat->block_header ) <= 0 ) 
-						syslog(LOG_ERR, "Ident: %s, failed to write xstat buffer to disk: '%s'" , fs->Ident, strerror(errno));
+						LogError("Ident: %s, failed to write xstat buffer to disk: '%s'" , fs->Ident, strerror(errno));
 
 					ResetPortHistogram(fs->xstat->port_histogram);
 					ResetBppHistogram(fs->xstat->bpp_histogram);
@@ -506,15 +505,15 @@ srecord_t	*commbuff;
 				if ( subdir && !SetupSubDir(fs->datadir, subdir, error, 255) ) {
 					// in this case the flows get lost! - the rename will fail
 					// but this should not happen anyway, unless i/o problems, inode problems etc.
-					syslog(LOG_ERR, "Ident: %s, Failed to create sub hier directories: %s", fs->Ident, error );
+					LogError("Ident: %s, Failed to create sub hier directories: %s", fs->Ident, error );
 				}
 
 				// if rename fails, we are in big trouble, as we need to get rid of the old .current file
 				// otherwise, we will loose flows and can not continue collecting new flows
 				err = rename(fs->current, nfcapd_filename);
 				if ( err ) {
-					syslog(LOG_ERR, "Ident: %s, Can't rename dump file: %s", fs->Ident,  strerror(errno));
-					syslog(LOG_ERR, "Ident: %s, Serious Problem! Fix manually", fs->Ident);
+					LogError("Ident: %s, Can't rename dump file: %s", fs->Ident,  strerror(errno));
+					LogError("Ident: %s, Serious Problem! Fix manually", fs->Ident);
 					if ( launcher_pid )
 						commbuff->failed = 1;
 
@@ -531,7 +530,7 @@ srecord_t	*commbuff;
 				}
 
 				// log stats
-				syslog(LOG_INFO,"Ident: '%s' Flows: %llu, Packets: %llu, Bytes: %llu, Sequence Errors: %u, Bad Packets: %u", 
+				LogInfo("Ident: '%s' Flows: %llu, Packets: %llu, Bytes: %llu, Sequence Errors: %u, Bad Packets: %u", 
 					fs->Ident, (unsigned long long)nffile->stat_record->numflows, (unsigned long long)nffile->stat_record->numpackets, 
 					(unsigned long long)nffile->stat_record->numbytes, nffile->stat_record->sequence_failure, fs->bad_packets);
 
@@ -576,14 +575,14 @@ srecord_t	*commbuff;
 					commbuff->subdir[0] = '\0';
 
 				if ( launcher_alive ) {
-					syslog(LOG_DEBUG, "Signal launcher");
+					LogInfo("Signal launcher");
 					kill(launcher_pid, SIGHUP);
 				} else 
-					syslog(LOG_ERR, "ERROR: Launcher did unexpectedly!");
+					LogError("ERROR: Launcher died unexpectedly!");
 
 			}
 			
-			syslog(LOG_INFO, "Total ignored packets: %u", ignored_packets);
+			LogInfo("Total ignored packets: %u", ignored_packets);
 			ignored_packets = 0;
 
 			if ( done )
@@ -611,7 +610,7 @@ srecord_t	*commbuff;
 				break;
 			else {
 				/* this should never be executed as it should be caught in other places */
-				syslog(LOG_ERR, "error condition in '%s', line '%d', cnt: %i", __FILE__, __LINE__ ,(int)cnt);
+				LogError("error condition in '%s', line '%d', cnt: %i", __FILE__, __LINE__ ,(int)cnt);
 				continue;
 			}
 		}
@@ -624,7 +623,7 @@ srecord_t	*commbuff;
 
 		fs = GetFlowSource(&sf_sender);
 		if ( fs == NULL ) {
-			syslog(LOG_WARNING, "Skip UDP packet. Ignored packets so far %u packets", ignored_packets);
+			LogError("Skip UDP packet. Ignored packets so far %u packets", ignored_packets);
 			ignored_packets++;
 			continue;
 		}
@@ -632,7 +631,7 @@ srecord_t	*commbuff;
 
 		/* check for too little data - cnt must be > 0 at this point */
 		if ( cnt < sizeof(common_flow_header_t) ) {
-			syslog(LOG_WARNING, "Ident: %s, Data length error: too little data for common netflow header. cnt: %i",fs->Ident, (int)cnt);
+			LogError("Ident: %s, Data length error: too little data for common netflow header. cnt: %i",fs->Ident, (int)cnt);
 			fs->bad_packets++;
 			continue;
 		}
@@ -651,7 +650,7 @@ srecord_t	*commbuff;
 			fs->nffile->block_header->size 		 = 0;
 			fs->nffile->block_header->NumRecords = 0;
 			fs->nffile->buff_ptr = (void *)((pointer_addr_t)fs->nffile->block_header + sizeof(data_block_header_t) );
-			syslog(LOG_ERR, "### Software bug ### Ident: %s, output buffer overflow: expect memory inconsitency", fs->Ident);
+			LogError("### Software bug ### Ident: %s, output buffer overflow: expect memory inconsitency", fs->Ident);
 		}
 	}
 
@@ -672,7 +671,7 @@ int main(int argc, char **argv) {
  
 char	*bindhost, *filter, *datadir, pidstr[32], *launch_process;
 char	*userid, *groupid, *checkptr, *listenport, *mcastgroup, *extension_tags;
-char	*Ident, *pcap_file, pidfile[MAXPATHLEN];
+char	*Ident, *pcap_file, *time_extension, pidfile[MAXPATHLEN];
 struct stat fstat;
 srecord_t	*commbuff;
 packet_function_t receive_packet;
@@ -681,7 +680,7 @@ FlowSource_t *fs;
 struct sigaction act;
 int		family, bufflen;
 time_t 	twin, t_start;
-int		sock, err, synctime, do_daemonize, expire, report_sequence, do_xstat;
+int		sock, err, synctime, do_daemonize, expire, spec_time_extension, report_sequence, do_xstat;
 int		subdir_index, compress;
 int	c;
 
@@ -702,8 +701,10 @@ int	c;
 	twin	 		= TIME_WINDOW;
 	datadir	 		= NULL;
 	subdir_index	= 0;
+	time_extension	= "%Y%m%d%H%M";
 	expire			= 0;
-	compress		= 0;
+	spec_time_extension = 0;
+	compress		= NOT_COMPRESSED;
 	do_xstat		= 0;
 	memset((void *)&peer, 0, sizeof(send_peer_t));
 	peer.family		= AF_UNSPEC;
@@ -712,7 +713,7 @@ int	c;
 	extension_tags	= DefaultExtensions;
 	pcap_file		= NULL;
 
-	while ((c = getopt(argc, argv, "46ewhEVI:DB:b:f:j:l:n:p:P:R:S:T:t:x:ru:g:z")) != EOF) {
+	while ((c = getopt(argc, argv, "46ewhEVI:DB:b:f:jl:n:p:J:P:R:S:T:t:x:ru:g:zZ")) != EOF) {
 		switch (c) {
 			case 'h':
 				usage(argv[0]);
@@ -763,8 +764,19 @@ int	c;
 			case 'w':
 				synctime = 1;
 				break;
+			case 'j':
+				if ( compress ) {
+					LogError("Use either -z for LZO or -j for BZ2 compression, but not both\n");
+					exit(255);
+				}
+				compress = BZ2_COMPRESSED;
+				break;
 			case 'z':
-				compress = 1;
+				if ( compress ) {
+					LogError("Use either -z for LZO or -j for BZ2 compression, but not both\n");
+					exit(255);
+				}
+				compress = LZO_COMPRESSED;
 				break;
 			case 'B':
 				bufflen = strtol(optarg, &checkptr, 10);
@@ -775,7 +787,7 @@ int	c;
 			case 'b':
 				bindhost = optarg;
 				break;
-			case 'j':
+			case 'J':
 				mcastgroup = optarg;
 				break;
 			case 'p':
@@ -846,6 +858,10 @@ int	c;
 			case 'x':
 				launch_process = optarg;
 				break;
+			case 'Z':
+				time_extension	= "%Y%m%d%H%M%z";
+				spec_time_extension = 1;
+				break;
 			case '4':
 				if ( family == AF_UNSPEC )
 					family = AF_INET;
@@ -868,7 +884,12 @@ int	c;
 		}
 	}
 
-	InitExtensionMaps(NULL);
+	if ( expire && spec_time_extension ) {
+		fprintf(stderr, "ERROR, -Z timezone extension breaks expire -e\n");
+		exit(255);
+	}
+
+	InitExtensionMaps(NO_EXTENSION_LIST);
 	SetupExtensionDescriptors(strdup(extension_tags));
 
 	if ( FlowSource == NULL && datadir == NULL ) {
@@ -981,7 +1002,7 @@ int	c;
 		pid_t pid = getpid();
 		int pidf  = open(pidfile, O_RDWR|O_TRUNC|O_CREAT, S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH);
 		if ( pidf == -1 ) {
-			syslog(LOG_ERR, "Error opening pid file: '%s' %s", pidfile, strerror(errno));
+			LogError("Error opening pid file: '%s' %s", pidfile, strerror(errno));
 			close(sock);
 			exit(255);
 		}
@@ -999,7 +1020,7 @@ int	c;
 		// prepare shared memory
 		shmem = mmap(0, sizeof(srecord_t), PROT_READ | PROT_WRITE, MAP_ANON | MAP_SHARED, -1, 0);
 		if ( shmem == (caddr_t)-1 ) {
-			syslog(LOG_ERR, "mmap() error: %s", strerror(errno));
+			LogError("mmap() error: %s", strerror(errno));
 			close(sock);
 			exit(255);
 		}
@@ -1015,7 +1036,7 @@ int	c;
 				exit(0);
 				break;
 			case -1:
-				syslog(LOG_ERR, "fork() error: %s", strerror(errno));
+				LogError("fork() error: %s", strerror(errno));
 				if ( strlen(pidfile) )
 					unlink(pidfile);
 				exit(255);
@@ -1023,14 +1044,14 @@ int	c;
 			default:
 				// parent
 			launcher_alive = 1;
-			syslog(LOG_DEBUG, "Launcher[%i] forked", launcher_pid);
+			LogInfo("Launcher[%i] forked", launcher_pid);
 		}
 	}
 
 	fs = FlowSource;
 	while ( fs ) {
 		if ( InitBookkeeper(&fs->bookkeeper, fs->datadir, getpid(), launcher_pid) != BOOKKEEPER_OK ) {
-			syslog(LOG_ERR, "initialize bookkeeper failed.");
+			LogError("initialize bookkeeper failed.");
 
 			// release all already allocated bookkeepers
 			fs = FlowSource;
@@ -1048,7 +1069,6 @@ int	c;
 
 		// Init the extension map list
 		if ( !InitExtensionMapList(fs) ) {
-			// error message goes to syslog
 			exit(255);
 		}
 
@@ -1066,8 +1086,9 @@ int	c;
 	sigaction(SIGALRM, &act, NULL);
 	sigaction(SIGCHLD, &act, NULL);
 
-	syslog(LOG_INFO, "Startup.");
-	run(receive_packet, sock, peer, twin, t_start, report_sequence, datadir, subdir_index, compress, do_xstat);
+	LogInfo("Startup.");
+	run(receive_packet, sock, peer, twin, t_start, report_sequence, subdir_index, 
+		time_extension, compress, do_xstat);
 	close(sock);
 	kill_launcher(launcher_pid);
 
@@ -1078,15 +1099,15 @@ int	c;
 		if ( expire == 0 && ReadStatInfo(fs->datadir, &dirstat, LOCK_IF_EXISTS) == STATFILE_OK ) {
 			UpdateBookStat(dirstat, fs->bookkeeper);
 			WriteStatInfo(dirstat);
-			syslog(LOG_INFO, "Updating statinfo in directory '%s'", datadir);
+			LogInfo("Updating statinfo in directory '%s'", datadir);
 		}
 
 		ReleaseBookkeeper(fs->bookkeeper, DESTROY_BOOKKEEPER);
 		fs = fs->next;
 	}
 
-	syslog(LOG_INFO, "Terminating sfcapd.");
-	closelog();
+	LogInfo("Terminating sfcapd.");
+	EndLog();
 
 	if ( strlen(pidfile) )
 		unlink(pidfile);
