@@ -1,4 +1,6 @@
 /*
+ *  Copyright (c) 2016, Peter Haag
+ *  Copyright (c) 2014, Peter Haag
  *  Copyright (c) 2009, Peter Haag
  *  Copyright (c) 2004-2008, SWITCH - Teleinformatikdienste fuer Lehre und Forschung
  *  All rights reserved.
@@ -11,7 +13,7 @@
  *   * Redistributions in binary form must reproduce the above copyright notice, 
  *     this list of conditions and the following disclaimer in the documentation 
  *     and/or other materials provided with the distribution.
- *   * Neither the name of SWITCH nor the names of its contributors may be 
+ *   * Neither the name of the author nor the names of its contributors may be 
  *     used to endorse or promote products derived from this software without 
  *     specific prior written permission.
  *  
@@ -135,7 +137,6 @@
 #include <netinet/in.h>
 #include <arpa/inet.h>
 #include <sys/time.h>
-#include <syslog.h>
 
 #ifdef HAVE_STDINT_H
 #include <stdint.h>
@@ -351,6 +352,9 @@ typedef struct _SFSample {
 	uint32_t samplePool;
 	uint32_t dropEvents;
 
+	/* exception handler context */
+	jmp_buf env;
+
 	/* the sampled header */
 	uint32_t packet_data_tag;
 	uint32_t headerProtocol;
@@ -460,9 +464,6 @@ typedef struct _SFSample {
 	uint32_t statsSamplingInterval;
 	uint32_t counterBlockVersion;
 
-	/* exception handler context */
-	jmp_buf env;
-
 #define SFABORT(s, r) longjmp((s)->env, (r))
 #define SF_ABORT_EOS 1
 #define SF_ABORT_DECODE_ERROR 2
@@ -486,12 +487,7 @@ static inline uint64_t getData64(SFSample *sample);
 
 static void writeCountersLine(SFSample *sample);
 
-#ifdef __SUNPRO_C
-static void receiveError(SFSample *sample, char *errm, int hexdump);
-#pragma does_not_return (receiveError)
-#else
 static void receiveError(SFSample *sample, char *errm, int hexdump) __attribute__ ((noreturn));
-#endif
 
 static inline void skipBytes(SFSample *sample, int skip);
 
@@ -505,8 +501,6 @@ static inline uint32_t getString(SFSample *sample, char *buf, int bufLen);
 
 static inline uint32_t getAddress(SFSample *sample, SFLAddress *address);
 
-static inline char *printTag(uint32_t tag, char *buf, int bufLen);
-
 static inline void skipTLVRecord(SFSample *sample, uint32_t tag, uint32_t len, char *description);
 
 static inline void readSFlowDatagram(SFSample *sample, FlowSource_t *fs);
@@ -521,6 +515,10 @@ static inline void readCountersSample_v2v4(SFSample *sample, FlowSource_t *fs);
 
 static inline void StoreSflowRecord(SFSample *sample, FlowSource_t *fs);
 
+#ifdef DEVEL
+static inline char *printTag(uint32_t tag, char *buf, int bufLen);
+#endif
+
 extern int verbose;
 
 
@@ -673,7 +671,7 @@ static void receiveError(SFSample *sample, char *errm, int hexdump)
 		printHex(sample->rawSample, sample->rawSampleLen, scratch, 6000, markOffset, 16);
 		hex = scratch;
 	}
-	syslog(LOG_ERR, "SFLOW: %s (source IP = %s) %s\n", msg, IP_to_a(sample->sourceIP.s_addr, ipbuf, 51), hex);
+	LogError("SFLOW: %s (source IP = %s) %s", msg, IP_to_a(sample->sourceIP.s_addr, ipbuf, 51), hex);
 
 	SFABORT(sample, SF_ABORT_DECODE_ERROR);
 
@@ -689,7 +687,7 @@ static void lengthCheck(SFSample *sample, char *description, u_char *start, int
 	uint32_t adjustedLen = ((len + 3) >> 2) << 2;
 	if(actualLen != adjustedLen) {
 		dbg_printf("%s length error (expected %d, found %d)\n", description, len, actualLen);
-		syslog(LOG_ERR, "SFLOW: %s length error (expected %d, found %d)\n", description, len, actualLen);
+		LogError("SFLOW: %s length error (expected %d, found %d)", description, len, actualLen);
 		SFABORT(sample, SF_ABORT_LENGTH_ERROR);
   }
 
@@ -915,6 +913,7 @@ static void decodeIPV4(SFSample *sample)
 			dbg_printf("IPFragmentOffset %u\n", sample->ip_fragmentOffset);
 		}
 		else {
+			dbg_printf("Unfragmented\n");
 			/* advance the pointer to the next protocol layer */
 			/* ip headerLen is expressed as a number of quads */
 			ptr += (ip.version_and_headerLen & 0x0f) * 4;
@@ -1031,11 +1030,14 @@ value32_t	*val;
 uint32_t bytes, j, id, ipsize, ip_flags;
 uint64_t _bytes, _packets, _t;	// tmp buffers
 
+	dbg_printf("StoreSflowRecord\n");
+
 	gettimeofday(&now, NULL);
 
-	// ignore fragments
-	if( sample->ip_fragmentOffset > 0 ) 
-		return;
+	if( sample->ip_fragmentOffset > 0 ) {
+		sample->dcd_sport = 0;
+		sample->dcd_dport = 0;
+	}
 
 	bytes = sample->sampledPacketSize;
 	
@@ -1052,11 +1054,11 @@ uint64_t _bytes, _packets, _t;	// tmp buffers
 	ip_flags &= IP_extension_mask;
 
 	if ( ip_flags >= MAX_SFLOW_EXTENSIONS ) {
-		syslog(LOG_ERR,"SFLOW: Corrupt ip_flags: %u", ip_flags);
+		LogError("SFLOW: Corrupt ip_flags: %u", ip_flags);
 	}
 	exporter = GetExporter(fs, sample->agentSubId, sample->meanSkipCount);
 	if ( !exporter ) {
-		syslog(LOG_ERR,"SFLOW: Exporter NULL: Abort sflow record processing");
+		LogError("SFLOW: Exporter NULL: Abort sflow record processing");
 		return;
 	}
 	exporter->packets++;
@@ -1064,13 +1066,13 @@ uint64_t _bytes, _packets, _t;	// tmp buffers
 	// get appropriate extension map
 	extension_map = exporter->sflow_extension_info[ip_flags].map;
 	if ( !extension_map ) {
-		syslog(LOG_INFO,"SFLOW: setup extension map: %u ", ip_flags);
+		LogInfo("SFLOW: setup extension map: %u", ip_flags);
 		if ( !Setup_Extension_Info(fs, exporter, ip_flags ) ) {
-			syslog(LOG_ERR,"SFLOW: Extension map: NULL: Abort sflow record processing");
+			LogError("SFLOW: Extension map: NULL: Abort sflow record processing");
 			return;
 		}
 		extension_map = exporter->sflow_extension_info[ip_flags].map;
-		syslog(LOG_INFO,"SFLOW: setup extension map: %u done.", ip_flags);
+		LogInfo("SFLOW: setup extension map: %u done", ip_flags);
 	}
 
 	// output buffer size check
@@ -1078,10 +1080,11 @@ uint64_t _bytes, _packets, _t;	// tmp buffers
 	ipsize = sample->gotIPV6 ? 32 : 8;
 	if ( !CheckBufferSpace(fs->nffile, sflow_output_record_size[ip_flags] + ipsize )) {
 		// fishy! - should never happen. maybe disk full?
-		syslog(LOG_ERR,"SFLOW: output buffer size error. Abort sflow record processing");
+		LogError("SFLOW: output buffer size error. Abort sflow record processing");
 		return;
 	}
 
+	dbg_printf("Fill Record\n");
 	common_record = (common_record_t *)fs->nffile->buff_ptr;
 
 	common_record->size			  = sflow_output_record_size[ip_flags] + ipsize;
@@ -1096,9 +1099,10 @@ uint64_t _bytes, _packets, _t;	// tmp buffers
 	common_record->last			  = common_record->first;
 	common_record->msec_first	  = now.tv_usec / 1000;
 	common_record->msec_last	  = common_record->msec_first;
-	_t							  = 1000*now.tv_sec + common_record->msec_first;	// tmp buff for first_seen
+	_t							  = 1000LL * now.tv_sec + common_record->msec_first;	// tmp buff for first_seen
 
 	common_record->fwd_status	  = 0;
+	common_record->reserved	  	  = 0;
 	common_record->tcp_flags	  = sample->dcd_tcpFlags;
 	common_record->prot			  = sample->dcd_ipProtocol;
 	common_record->tos			  = sample->dcd_ipTos;
@@ -1242,12 +1246,11 @@ uint64_t _bytes, _packets, _t;	// tmp buffers
 			case EX_RECEIVED: {
 				tpl_ext_27_t *tpl = (tpl_ext_27_t *)next_data;
 				tpl->received  = (uint64_t)((uint64_t)fs->received.tv_sec * 1000LL) + (uint64_t)((uint64_t)fs->received.tv_usec / 1000LL);
-printf("t-received: %llu\n", tpl->received);
 				next_data = (void *)tpl->data;
 			} break;
 			default: 
 				// this should never happen
-				syslog(LOG_ERR,"SFLOW: Unexpected extension %i for sflow record. Skip extension", id);
+				LogError("SFLOW: Unexpected extension %i for sflow record. Skip extension", id);
 				dbg_printf("SFLOW: Unexpected extension %i for sflow record. Skip extension", id);
 		}
 		j++;
@@ -1404,13 +1407,14 @@ static inline uint32_t getAddress(SFSample *sample, SFLAddress *address) {
 	return address->type;
 } // End of getAddress
 
+static inline void skipTLVRecord(SFSample *sample, uint32_t tag, uint32_t len, char *description) {
+
+#ifdef DEVEL
 static inline char *printTag(uint32_t tag, char *buf, int bufLen) {
 	snprintf(buf, bufLen, "%u:%u", (tag >> 12), (tag & 0x00000FFF));
 	return buf;
 } // End of printTag
 
-static inline void skipTLVRecord(SFSample *sample, uint32_t tag, uint32_t len, char *description) {
-#ifdef DEVEL
 char buf[51];
 #endif
 
@@ -1878,7 +1882,7 @@ static void readFlowSample_header(SFSample *sample) {
 		dbg_printf("NO_DECODE headerProtocol=%d\n", sample->headerProtocol);
 		break;
 	default:
-		syslog(LOG_ERR, "SFLOW: undefined headerProtocol = %d\n", sample->headerProtocol);
+		LogError("SFLOW: undefined headerProtocol = %d", sample->headerProtocol);
 		exit(-12);
 	}
 	
@@ -2046,15 +2050,24 @@ static void readFlowSample_v2v4(SFSample *sample, FlowSource_t *fs) {
 			uint32_t extended_tag;
 			extended_tag = getData32(sample);
 			switch(extended_tag) {
-			case INMEXTENDED_SWITCH: readExtendedSwitch(sample); break;
-			case INMEXTENDED_ROUTER: readExtendedRouter(sample); break;
+			case INMEXTENDED_SWITCH: 
+				readExtendedSwitch(sample); break;
+			case INMEXTENDED_ROUTER: 
+				readExtendedRouter(sample); break;
 			case INMEXTENDED_GATEWAY:
-	if(sample->datagramVersion == 2) readExtendedGateway_v2(sample);
-	else readExtendedGateway(sample);
-	break;
-			case INMEXTENDED_USER: readExtendedUser(sample); break;
-			case INMEXTENDED_URL: readExtendedUrl(sample); break;
-			default: receiveError(sample, "unrecognized extended data tag", YES); break;
+				if(sample->datagramVersion == 2) 
+					readExtendedGateway_v2(sample);
+				else 
+					readExtendedGateway(sample);
+				break;
+			case INMEXTENDED_USER: 
+				readExtendedUser(sample); break;
+			case INMEXTENDED_URL: 
+				readExtendedUrl(sample); break;
+			default: 
+				LogError("Unrecognized extended data tag: %u", extended_tag); 
+				receiveError(sample, "unrecognized extended data tag", YES); 
+			break;
 			}
 		}
 	}
@@ -2465,6 +2478,9 @@ char buf[51];
 
 	/* now iterate and pull out the flows and counters samples */
 	for(; samp < samplesInPacket; samp++) {
+		memset(&sample->packet_data_tag, 0,
+			sizeof(*sample) - ((void *)&sample->packet_data_tag - (void *)sample));
+
 		// just read the tag, then call the approriate decode fn
 		sample->sampleType = getData32(sample);
 		dbg_printf("startSample ----------------------\n");
@@ -2547,7 +2563,7 @@ int Setup_Extension_Info(FlowSource_t *fs, exporter_sflow_t	*exporter, int num)
 int i, id, extension_size, map_size, map_index;
 
 	dbg_printf("Setup Extension ID 0x%x\n", num);
-	syslog(LOG_INFO, "SFLOW: setup extension map %u", num);
+	LogInfo("SFLOW: setup extension map %u", num);
 
 	// prepare sflow extension map <num>
 	exporter->sflow_extension_info[num].map   = NULL;
@@ -2560,10 +2576,11 @@ int i, id, extension_size, map_size, map_index;
 	if ( ( map_size & 0x3 ) != 0 )
 		map_size += 2;
 
+
 	// Create a generic sflow extension map
 	exporter->sflow_extension_info[num].map = (extension_map_t *)malloc((size_t)map_size);
 	if ( !exporter->sflow_extension_info[num].map ) {
-		syslog(LOG_ERR, "SFLOW: malloc() error in %s line %d: %s\n", __FILE__, __LINE__, strerror (errno));
+		LogError("malloc() allocation error in %s line %d: %s", __FILE__, __LINE__, strerror(errno) );
 		return 0;
 	}
 
@@ -2613,8 +2630,8 @@ int i, id, extension_size, map_size, map_index;
 	exporter->sflow_extension_info[num].map->map_id   	  = INIT_ID;		
 	exporter->sflow_extension_info[num].map->extension_size = extension_size;		
 
-	syslog(LOG_INFO, "Extension size: %i\n", extension_size);
-	syslog(LOG_INFO, "Extension map size: %i\n", map_size);
+	LogInfo("Extension size: %i", extension_size);
+	LogInfo("Extension map size: %i", map_size);
 
 	if ( !AddExtensionMap(fs, exporter->sflow_extension_info[num].map) ) {
 		// bad - we must free this map and fail - otherwise data can not be read any more
@@ -2623,7 +2640,7 @@ int i, id, extension_size, map_size, map_index;
 		return 0;
 	}
 	dbg_printf("New Extension map ID %i\n", exporter->sflow_extension_info[num].map->map_id);
-	syslog(LOG_INFO, "New extension map id: %i\n", exporter->sflow_extension_info[num].map->map_id);
+	LogInfo("New extension map id: %i", exporter->sflow_extension_info[num].map->map_id);
 
 	return 1;
 
@@ -2657,11 +2674,11 @@ int i;
 	}
 
 	// nothing found
-	syslog(LOG_INFO, "SFLOW: New exporter\n" );
+	LogInfo("SFLOW: New exporter" );
 
 	*e = (exporter_sflow_t *)malloc(sizeof(exporter_sflow_t));
 	if ( !(*e)) {
-		syslog(LOG_ERR, "SFLOW: malloc() error in %s line %d: %s\n", __FILE__, __LINE__, strerror (errno));
+		LogError("SFLOW: malloc() error in %s line %d: %s", __FILE__, __LINE__, strerror (errno));
 		return NULL;
 	}
 	memset((void *)(*e), 0, sizeof(exporter_sflow_t));
@@ -2681,7 +2698,7 @@ int i;
 
 	sampler = (generic_sampler_t *)malloc(sizeof(generic_sampler_t));
 	if ( !sampler ) {
-		syslog(LOG_ERR, "SFLOW: malloc() error in %s line %d: %s\n", __FILE__, __LINE__, strerror (errno));
+		LogError("SFLOW: malloc() error in %s line %d: %s", __FILE__, __LINE__, strerror (errno));
 		return NULL;
 	}
 	(*e)->sampler = sampler;
@@ -2699,7 +2716,7 @@ int i;
 
 	dbg_printf("SFLOW: New exporter: SysID: %u, agentSubId: %u, MeanSkipCount: %u, IP: %s\n", 
 		(*e)->info.sysid, agentSubId, meanSkipCount, ipstr);
-	syslog(LOG_INFO, "SFLOW: New exporter: SysID: %u, agentSubId: %u, MeanSkipCount: %u, IP: %s\n", 
+	LogInfo("SFLOW: New exporter: SysID: %u, agentSubId: %u, MeanSkipCount: %u, IP: %s", 
 		(*e)->info.sysid, agentSubId, meanSkipCount, ipstr);
 
 	return (*e);
@@ -2725,7 +2742,7 @@ int 		exceptionVal;
 	} else {
 		// CATCH
 		dbg_printf("SFLOW: caught exception: %d\n", exceptionVal);
-		syslog(LOG_ERR, "SFLOW: caught exception: %d\n", exceptionVal);
+		LogError("SFLOW: caught exception: %d", exceptionVal);
 	}
 	dbg_printf("endDatagram	 =================================\n");
 
